/*************************************************************************
*
*  File Name   : Vidplay.c
*
*  Description : This program demonstrates the coordinated use of the
*                overlay, amp/mixer and videodisc devices.
*
*                Each is controlled via MCI messages directly to the device
*                and indirectly through Standard Multimedia Controls.
*
*                The application peforms the following functions:
*                    1) Create the main application window and menu
*                    2) Open the overlay, amp/mixer and videodisc devices
*                    3) Determine the transparency color used by the
*                       overlay device
*
*                    4) Validate the menu selections based on whether
*                       the devices were opened, and whether an element
*                       buffer has been allocated via Load or Capture.
*
*                User selections provide the following:
*                    1) Create a dialog for Audio Attributes Control
*                    2) Create a dialog for Video Attributes Control
*                    3) Create a dialog for Video Player Control
*
*                    4) Load an image element from a file
*                    5) Save an image element to a file
*
*                    6) Capture video from the screen to an image element
*                    6) Restore an image from an element to the screen
*
*  Concepts    :
*
*       Overlay device control
*       Amp/Mixer device control
*       Videodisc device control
*           Open, close, alias use, etc for each device
*
*       Audio Attributes Control usage
*       Video Attributes Control usage
*       Player Device Control usage
*           Creation, sizing, device attachment, etc for each control
*
*       Image Capture/Restore (to and from the screen)
*       Image Load/Save (to and from a file)
*
*  Copyright (C) 1992 IBM Corporation
*
*      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
*      sample code created by IBM Corporation. This sample code is not
*      part of any standard or IBM product and is provided to you solely
*      for  the purpose of assisting you in the development of your
*      applications.  The code is provided "AS IS", without
*      warranty of any kind.  IBM shall not be liable for any damages
*      arising out of your use of the sample code, even if they have been
*      advised of the possibility of such damages.                                                    *
*
************************************************************************/

#define INCL_WIN
#define INCL_WINSYS                     /* Selectively include PM parts */
#define INCL_WINFRAMEMGR
#define INCL_WINMENUS
#define INCL_WINDIALOGS
#define INCL_WINSTDDLGS
#define INCL_DOSMODULEMGR
#define INCL_GPI
#define INCL_MMCTL

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <os2me.h>
#include "vidplay.h"
#include "MMCTL.H"

#define LENGTH_STRING  33               /* Length of string             */
#define RETURN_ERROR    1               /* return value for exit        */
#define BEEP_WARN_FREQ      60          /* frequency of warning beep    */
#define BEEP_WARN_DUR      100          /* duration of warning beep     */
#define X_MMCTL  8                      /* X offset of MM control       */
#define Y_MMCTL  3                      /* Y offset of MM control       */

/************************************************/
/********** Function Prototype Section **********/
/************************************************/
int main( VOID );
MRESULT EXPENTRY MainWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY ProdInfoDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY VidplayHelpInfoDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY pAACDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY pVACDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY pVPCDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 );
VOID OptimizeControlDialogSize (HWND hwndControlDlg, HWND hwndControl);
USHORT  SetupMyMainWindow (VOID);
DWORD CreateControls (VOID);
VOID DisplayMessage(PCH str);

DWORD GetElementName (HWND hwnd, PSZ pszTitle, PSZ pszElementName);

/****************************************************************/
/* MCI Device Operation functions -                             */
/*  These package MCI calls for easy debugging & readability    */
/****************************************************************/
DWORD OpenDevices (VOID);
DWORD OpenDevice (LPSTR lpstrDevName, LPSTR lpstrDevAlias, WORD *pwDeviceID);
DWORD CloseDevice (WORD wDeviceID);
DWORD AcquireDevice (WORD wDeviceID, DWORD dwExclusive);
DWORD LoadElement (WORD wDeviceID, PSZ pszName);
DWORD SaveElement (WORD wDeviceID, DWORD dwFlag, PSZ pszName);
DWORD Capture (WORD wDeviceID, DWORD dwFlag,
                   DWORD dwX1, DWORD dwY1,
                   DWORD dwX2, DWORD dwY2);
DWORD Restore (WORD wDeviceID, DWORD dwFlag,
                   DWORD dwX1, DWORD dwY1,
                   DWORD dwX2, DWORD dwY2);
DWORD CheckUnFreeze (WORD wDeviceID, DWORD dwFlag,
                     DWORD dwX1, DWORD dwY1, DWORD dwX2, DWORD dwY2);
DWORD UseAlternateWindow (WORD wDeviceID, HWND hwndAlternate);
DWORD CheckTransparencyColor (WORD wDeviceID, COLOR *TransColor);
DWORD SelectSource (WORD wDeviceID, DWORD dwSourceNum,
                    DWORD dwSourceType, DWORD dwSetting);

/**************************** Global Variables. ****************************/
/* Define parameters by type */
HAB     hab;
HMQ     hmq;
HWND    hwndFrame;               /* Frame window handle */
HWND    hwndClient;              /* Client window handle-Parent of Notebook */
HWND    hwndControlClient;       /* Client window handle-Parent of Notebook */
HWND    hwndMainDlg;             /* Handle of window containing the Notebook */
HWND    hwndMenu;                /* Handle for Menu           */
HMODULE hMMCTLDll;                    /* MMCTL.DLL handle      */
HWND    hwndAACDlg;                 /* Handle for Audio control */
HWND    hwndVACDlg;                 /* Handle for Video control */
HWND    hwndVPCDlg;                 /* Handle for Player control */
HWND    hwndAAC;                 /* Handle for Audio control */
HWND    hwndVAC;                 /* Handle for Video control */
HWND    hwndVPC;                 /* Handle for Player control */


WORD wOverlayDevID = 0;
WORD wAmpMixDevID  = 0;
WORD wVdiscDevID   = 0;

/* Device Names and Alias used for MCI control */
/* These are NOT displayed anywhere, and do NOT have to be translated ! */
static szOverlayDevName [] = "Overlay";

/* This version of code opens ONLY the M-Motion Amp Mixer.  To use
   this with other amp/mixers use the name "Ampmix".  Other amp mixers
   MUST default to pass-through audio enabled.  Some do not. */
/* static szAmpMixDevName  [] = "Ampmix"; */
static szAmpMixDevName  [] = "M-Motion Amp/Mixer";

static szVdiscDevName   [] = "Videodisc";
static szOverlayAlias[] = "VidplayOV1";
static szAmpMixAlias [] = "VidplayAM1";
static szVdiscAlias  [] = "VidplayVD1";
BOOL   fOverlayAcquired = FALSE;
BOOL   fVdiscAcquired   = FALSE;
BOOL   fAmpMixAcquired  = FALSE;

DWORD dwInformType = MCI_WAIT;
BOOL  fElement = FALSE;
HWND  hwndNotify = 0;
HWND  hwndDefaultVW = 0;
HWND  hwndAltVW     = 0;

DWORD dwAACShape = MCS_COMPLETE;
DWORD dwVACShape = MCS_COMPLETE;
DWORD dwVPCShape = MCS_COMPLETE;

COLOR TransColor = CLR_BLACK;

/************************************************/
/****** Structure Initialization Section ********/
/************************************************/


/************************ End Global Variables. ****************************/

/****************************************************************
 *  Name:   Main
 *
 *  Description  :
 *      Main PM program block which sets up the main window, queue, etc
 *
 *  Concepts     :
 *      Standard PM programming.
 *
 *  Parameters   : None.
 *
 *  Returns      :  Returns 0 to the operating system.
 *
 ****************************************************************/
int main( VOID )
   {
   QMSG         qmsg;
   BOOL         success;
   APIRET       aRC;
   CHAR         szFailBuffer[128];

   /* Initialize and get an anchor block handle */
   hab = WinInitialize( 0 );
   if ( !hab )
      {
      DosBeep( BEEP_WARN_FREQ, BEEP_WARN_DUR );
      exit( RETURN_ERROR );
      } /* endif */

   aRC = DosLoadModule(szFailBuffer, sizeof(szFailBuffer), MMCONTROLS_DLLNAME, &hMMCTLDll);
   if(aRC != 0)
      return FALSE;

   /* Create a message queue */
   hmq = WinCreateMsgQueue( hab, 0 );
   if ( !hmq)
      {
      DosBeep( BEEP_WARN_FREQ, BEEP_WARN_DUR );
      exit( RETURN_ERROR );
      } /* endif */

   /* Display the Product Infomation dialog box */
   success = WinDlgBox( HWND_DESKTOP,
                        HWND_DESKTOP,
                        ( PFNWP )ProdInfoDlgProc,
                        ( HMODULE ) 0,
                        ID_DLG_PRODINFO,
                        NULL );

   if ( !success )
      {
      DisplayMessage( "Could Not Register Class" );
      exit( RETURN_ERROR );
      } /* endif */

   /* This function handles the main window set-up */
   SetupMyMainWindow ();

   /***************************************************************
   * Get and dispatch messages from the application message queue
   * until WinGetMsg returns FALSE, indicating a WM_QUIT message.
   ****************************************************************/
   while ( WinGetMsg( hab, &qmsg, ( HWND )NULL, 0, 0) )
      WinDispatchMsg( hab, &qmsg );

   WinDestroyWindow( hwndFrame );      /* Destroy main window */
   WinDestroyMsgQueue( hmq );          /* Destroy message queue */
   WinTerminate( hab );                /* Terminate application */
   return( 0 );

   }   /* End main()  */

/***********************  End of main procedure  ************************/

/*************************************************************************
 *  Name:   SetupMyMainWindow
 *
 *  Description :
 *      This function will perform the initial window and device setup
 *      operations requried to begin communicating with the devices.
 *      If the device opening process fails for any reason, the program
 *      will exit back to OS/2.
 *
 *  Concepts :
 *      Procedure for setting up the Main Window.
 *      Registers and creates the main window
 *      Opens the devices
 *      Instructs the Overlay device to use the Client area
 *            as the video window
 *      Determines the transparency color used by the Overlay
 *            device
 *      Sets the background of the video window to the
 *            transparency color
 *
 *  Parameters   : None.
 *
 *  Returns:
 *     FALSE if ALL OK
 *     > 0 if an error occured that can be bypassed
 *************************************************************************/
USHORT SetupMyMainWindow(VOID)
   {
   BOOL success;
   DWORD dwRC;
   ULONG    flCreate = 0;
   HPOINTER hptrOriginal;           /* Handle to the default system cursor */
   HPOINTER hptrWait;               /* Handle to the system wait cursor */
   HPOINTER hptrCurrent;            /* Handle to the cursor in use */
   ULONG    cxSizeBorder = WinQuerySysValue(HWND_DESKTOP, SV_CXSIZEBORDER);
   ULONG    cySizeBorder = WinQuerySysValue(HWND_DESKTOP, SV_CYSIZEBORDER);
   ULONG    cyTitleBar   = WinQuerySysValue(HWND_DESKTOP, SV_CYTITLEBAR);
   ULONG    cyMenu       = WinQuerySysValue(HWND_DESKTOP, SV_CYMENU);


   /* Set Frame window flags */
   flCreate = FCF_STANDARD;                  /* Frame Control Flags */

   success = WinRegisterClass( hab,            /* Register Window class */
                         "MainWindow",
                         MainWinProc,
                         CS_SIZEREDRAW | CS_MOVENOTIFY,
                         0 );

   if ( !success )
      {
      DisplayMessage( "Could Not Register Class" );
      exit( RETURN_ERROR );
      } /* endif */

   /* Create main frame window */
   hwndFrame = WinCreateStdWindow( HWND_DESKTOP,     /* Desktop window is parent */
                                0,                   /* Invisible until resized  */
                                &flCreate,
                                "MainWindow",
                                "",
                                0L,
                                ( HMODULE )NULL,
                                ID_MAINWND,
                                &hwndClient );       /* Client Window Handle */

   if ( !hwndFrame )
      {
      DisplayMessage( "Could Not Create Main Window" );
      exit( RETURN_ERROR );
      } /* endif */

   /* Make this window 1/4 of the standard 640x480 screen - this
      is close to the ideal ration for the video. Allow room for
      size borders, menu bar, and title bar */
   WinSetWindowPos (hwndFrame, 0, 0, 0, 
      320 + 2 * cxSizeBorder,
      240 + 2 * cySizeBorder + cyTitleBar + cyMenu, SWP_SIZE | SWP_SHOW);

   /* Use the frame window handle as the destination for MCI
      notify messages */
   hwndNotify = hwndFrame;

   /* Set Frame window title text */
   WinSetWindowText( hwndFrame, "Sample VIDPLAY" );

   /* Obtain handle of system pointer (cursor) */
   hptrOriginal = WinQueryPointer( HWND_DESKTOP );

   /* Obtain handle of system wait pointer (cursor) */
   hptrWait = WinQuerySysPointer( HWND_DESKTOP, SPTR_WAIT, FALSE );
   WinSetPointer( HWND_DESKTOP, hptrWait );
   hptrCurrent = hptrWait;

   /* Open the various MCI devices - OVERLAY, AMP/MIXER, Videodisc */
   if (!OpenDevices ())
      {
      /* Create the initial versions of the Standard Multimedia Controls */
      CreateControls ();

      /* Use the Client area as the window in which video is to be displayed */
      dwRC = UseAlternateWindow (wOverlayDevID, hwndClient);
      if (LOUSHORT(dwRC) != MCIERR_SUCCESS)
         {
         DisplayMessage ("Could not set Overlay to use Client window");
         exit( RETURN_ERROR );
         }

      WinSetPointer( HWND_DESKTOP, hptrOriginal );

      /* Query transparency color to be used to paint client window
       * background to enable video display
       */
      if (CheckTransparencyColor (wOverlayDevID, &TransColor))
         {
         TransColor = CLR_PINK;
         }

      /* Cause client window background to be repainted */
      WinInvalidateRect(hwndFrame, NULL, TRUE);
      }
   else
      {
      DisplayMessage ("Device(s) not available.  Program terminating");
      exit( RETURN_ERROR );
      }


   return ( FALSE );
   }  /* End SetupMyMainWindow() */
/**************  End of SetupMyMainWindow procedure  ********************/

/************************************************************************
 * Function : OpenDevices
 *
 * Description :
 *      This function will attempt to open the overlay, amp/mixer and
 *      videodisc player devices.
 *
 *      If the overlay or amp/mixer devices cannot be opened, the
 *      function will return with an error, and the program will
 *      terminate.
 *
 *      NOTE : Lack of videodisc is NOT considered an error,
 *             since a live camera or other video source may be used.
 * Parameters :
 *      None
 *
 * Returns :
 *      0 if ALL OK
 *      > 0 if an error occurred
 ************************************************************************/
DWORD OpenDevices ()
   {
   /* Open the M-MOTION OVERLAY device */
   if (OpenDevice ((LPSTR)szOverlayDevName, (LPSTR)szOverlayAlias, &wOverlayDevID))
      {
      DisplayMessage ("Overlay device not available!");
      return (TRUE);
      }

   /* Open the M-MOTION AMP MIXER device */
   if (OpenDevice ((LPSTR)szAmpMixDevName, (LPSTR)szAmpMixAlias, &wAmpMixDevID))
      {
      DisplayMessage ("Amp/Mixer device not available!");
      }

   /* Open the VIDEODISC PLAYER device */
   if (OpenDevice ((LPSTR)szVdiscDevName, (LPSTR)szVdiscAlias, &wVdiscDevID))
      {
      DisplayMessage ("Videodisc device not available!");
      }

   return (FALSE);
   }

/*************************************************************************
 *  Name:   CreateControls
 *
 *  Description :
 *      This function will create the dialogs (non-modal) that will
 *      contain the Audio Attributes Control, Video Attributes Control
 *      and Player Control.  The respective dialog procs for each of the
 *      dialogs will create a window for the actual control and attach
 *      the control to the device.
 *
 *      If the dialogs are created, each will be sized to the optimum
 *      required for the multimedia control it contains.
 *
 *  Concepts :
 *      Creates a dialog to hold a Multimedia Control
 *
 *  Parameters :
 *      None
 *
 *  Returns:
 *      0 if all OK
 *      > 0 if error
 *
 ****************************************************************/
DWORD CreateControls (VOID)
   {
   /* Create the Audio Attributes Control Dialog (prefix AAC) */
   if (wAmpMixDevID)
      {
      dwAACShape = MCS_MINIMAL;
      hwndAACDlg = WinLoadDlg(HWND_DESKTOP,
                           hwndClient,
                           pAACDlgProc,
                           0,
                           ID_DLG_AAC,
                           NULL);
      }
   else
      hwndAACDlg = 0;

   /* Create the Video Attributes Control Dialog (prefix VAC) */
   dwVACShape = MCS_MINIMAL;
   hwndVACDlg = WinLoadDlg(HWND_DESKTOP,
                        hwndClient,
                        pVACDlgProc,
                        0,
                        ID_DLG_VAC,
                        NULL);

   if (wVdiscDevID)
      {
      /* Create the Player Control Dialog (prefix VPC) */
      dwVPCShape = MCS_MINIMAL;
      hwndVPCDlg = WinLoadDlg(HWND_DESKTOP,
                           hwndClient,
                           pVPCDlgProc,
                           0,
                           ID_DLG_VPC,
                           NULL);
     }
   else
      hwndVPCDlg = 0;

   /* Were all the windows created?  If not, then return an error */
   if (!hwndVACDlg)
      return (TRUE);

   if (wAmpMixDevID && (!hwndAACDlg))
      return (TRUE);

   if (wVdiscDevID && (!hwndVPCDlg))
      return (TRUE);

   /* Optimize the sizes of the control dialogs */
   if (hwndAACDlg)
      OptimizeControlDialogSize (hwndAACDlg, hwndAAC);
   if (hwndVACDlg)
      OptimizeControlDialogSize (hwndVACDlg, hwndVAC);

   if (hwndVPCDlg)
      OptimizeControlDialogSize (hwndVPCDlg, hwndVPC);

   return (FALSE);
   }

/*************************************************************************
 *  Name:   MainWndProc
 *
 *  Description :
 *      Window procedure for the main clent window.
 *
 *  Concepts :
 *      Processes the messages sent to the main client
 *      window.  This routine processes the basic
 *      messages all client windows should process and
 *      passes all others onto UserWndProc where the
 *      developer can process any others.
 *
 *  Parameters   : hwnd - Window handle to which message is addressed
 *                msg - Message type
 *                mp1 - First message parameter
 *                mp2 - Second message parameter
 *
 *  Returns:  Return values are determined by each message
 *
 ****************************************************************/
MRESULT EXPENTRY MainWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {
   USHORT   usCommand;              /* WM_COMMAND command value */
   char    szTempElementName[256];

   switch ( msg )
      {
      case WM_CREATE:
          WinEnableMenuItem(hwndMenu, IDM_AUDIO, FALSE);
          break;

      case WM_INITMENU:
          WinEnableMenuItem((HWND)mp2, IDM_SAVE, fElement);
          WinEnableMenuItem((HWND)mp2, IDM_SAVEAS, fElement);

          WinEnableMenuItem((HWND)mp2, IDM_AUDIO, (BOOL)wAmpMixDevID);
          WinEnableMenuItem((HWND)mp2, IDM_AAC_MIN_SET, (BOOL)wAmpMixDevID);
          WinEnableMenuItem((HWND)mp2, IDM_AAC_COMP_SET, (BOOL)wAmpMixDevID);

          WinEnableMenuItem((HWND)mp2, IDM_PLAYER, (BOOL)wAmpMixDevID);
          WinEnableMenuItem((HWND)mp2, IDM_VPC_MIN_SET, (BOOL)wVdiscDevID);
          WinEnableMenuItem((HWND)mp2, IDM_VPC_COMP_SET, (BOOL)wVdiscDevID);


          WinEnableMenuItem((HWND)mp2, IDM_AUDIOSOURCE1, (BOOL)wAmpMixDevID);
          WinEnableMenuItem((HWND)mp2, IDM_AUDIOSOURCE2, (BOOL)wAmpMixDevID);

          WinEnableMenuItem((HWND)mp2, IDM_RESTORE, fElement);

          break;

      case WM_HELP:           /*  Display Help dialog when F1 is pressed */
         WinDlgBox(HWND_DESKTOP, hwnd,
               VidplayHelpInfoDlgProc,
               (HMODULE) 0, ID_HELP_INFO_DLG,
               0);
         break;

      /************************************************************/
      /************************************************************/
      /* Menu Item Selections                                     */
      /************************************************************/
      /************************************************************/
      case WM_COMMAND:
         usCommand = SHORT1FROMMP (mp1);  // Extract the command value
         switch (usCommand)
            {
            /**************************************/
            /****     FILE SUBMENU OPTIONS     ****/
            /**************************************/
            case IDM_LOAD:
               /* Load a image from a file into the image element buffer */
               if (!GetElementName (HWND_DESKTOP,
                                   (PSZ)"Load Element (name)",
                                   szTempElementName))
                  {
                  fElement = FALSE;
                  if (LoadElement (wOverlayDevID, szTempElementName))
                     DisplayMessage ("File requested could not be loaded");
                  else
                     /* there is now an element to use for save/restore */
                     fElement = TRUE;
                  }
               break;

            case IDM_SAVE:
               /* Save the current image element buffer to a file */
               if (fElement)
                  {
                  if (SaveElement (wOverlayDevID, 0, 0))
                     DisplayMessage ("Image could not be saved");
                  }
               else
                  DisplayMessage ("Perform a Load or Capture before saving");
               break;

            case IDM_SAVEAS:
               /* Save the current image element buffer to a named file */
               if (fElement)
                  {
                  if (!GetElementName (HWND_DESKTOP,
                                      (PSZ)"Save Element",
                                      szTempElementName))
                     {
                     if (SaveElement (wOverlayDevID, MCI_SAVE_FILE, szTempElementName))
                        DisplayMessage ("Image could not be saved");
                     }
                  }
               else
                  DisplayMessage ("Perform a Load or Capture before saving");
               break;

            case IDM_EXIT:
               WinPostMsg( hwnd, WM_CLOSE, NULL, NULL);
               break;

            /******************************************/
            /****     CONTROLS SUBMENU OPTIONS     ****/
            /******************************************/
            case IDM_AAC_MIN_SET:
               if (!wAmpMixDevID)
                  {
                  DisplayMessage ("No amp/mixer device available");
                  break;
                  }

               /* Create a MINMAL Audio Attributes Control */

               /* If it is already minimal, just show it */
               if (hwndAACDlg && (dwAACShape & MCS_MINIMAL))
                  {
                  WinSetWindowPos (hwndAACDlg, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
                  }
               else
                  {
                  if (hwndAACDlg)
                     WinDestroyWindow(hwndAACDlg);

                  dwAACShape = MCS_MINIMAL;
                  hwndAACDlg = WinLoadDlg(HWND_DESKTOP,
                                       hwndClient,
                                       pAACDlgProc,
                                       0,
                                       ID_DLG_AAC,
                                       NULL);
                  OptimizeControlDialogSize (hwndAACDlg, hwndAAC);
                  }
               break;

            case IDM_AAC_COMP_SET:
               if (!wAmpMixDevID)
                  {
                  DisplayMessage ("No amp/mixer device available");
                  break;
                  }

               /* Create a COMPLETE Audio Attributes Control */

               /* If it is already minimal, just show it */
               if (hwndAACDlg && (dwAACShape & MCS_COMPLETE))
                  {
                  WinSetWindowPos (hwndAACDlg, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
                  }
               else
                  {
                  if (hwndAACDlg)
                     WinDestroyWindow(hwndAACDlg);

                  dwAACShape = MCS_COMPLETE;
                  hwndAACDlg = WinLoadDlg(HWND_DESKTOP,
                                       hwndClient,
                                       pAACDlgProc,
                                       0,
                                       ID_DLG_AAC,
                                       NULL);
                  OptimizeControlDialogSize (hwndAACDlg, hwndAAC);
                  }
               break;

            case IDM_VAC_MIN_SET:
               /* Create a MINIMAL Video Attributes Control */

               /* If it is already minimal, just show it */
               if (hwndVACDlg && (dwVACShape & MCS_MINIMAL))
                  {
                  WinSetWindowPos (hwndVACDlg, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
                  }
               else
                  {
                  if (hwndVACDlg)
                     WinDestroyWindow(hwndVACDlg);

                  dwVACShape = MCS_MINIMAL;
                  hwndVACDlg = WinLoadDlg(HWND_DESKTOP,
                                       hwndClient,
                                       pVACDlgProc,
                                       0,
                                       ID_DLG_VAC,
                                       NULL);

                  OptimizeControlDialogSize (hwndVACDlg, hwndVAC);
                  }
               break;

            case IDM_VAC_COMP_SET:
               /* Create a COMPLETE Video Attributes Control */

               /* If it is already minimal, just show it */
               if (hwndVACDlg && (dwVACShape & MCS_COMPLETE))
                  {
                  WinSetWindowPos (hwndVACDlg, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
                  }
               else
                  {
                  if (hwndVACDlg)
                     WinDestroyWindow(hwndVACDlg);

                  dwVACShape = MCS_COMPLETE;
                  hwndVACDlg = WinLoadDlg(HWND_DESKTOP,
                                       hwndClient,
                                       pVACDlgProc,
                                       0,
                                       ID_DLG_VAC,
                                       NULL);
                  OptimizeControlDialogSize (hwndVACDlg, hwndVAC);
                  }
               break;

            case IDM_VPC_MIN_SET:
               if (!wVdiscDevID)
                  {
                  DisplayMessage ("No videodisc device available");
                  break;
                  }

               /* Create a MINIMAL Video Player Control */

               /* If it is already minimal, just show it */
               if (hwndVPCDlg && (dwVPCShape & MCS_MINIMAL))
                  {
                  WinSetWindowPos (hwndVPCDlg, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
                  }
               else
                  {
                  if (hwndVPCDlg)
                     WinDestroyWindow(hwndVPCDlg);

                  dwVPCShape = MCS_MINIMAL;
                  hwndVPCDlg = WinLoadDlg(HWND_DESKTOP,
                                       hwndClient,
                                       pVPCDlgProc,
                                       0,
                                       ID_DLG_VPC,
                                       NULL);

                  OptimizeControlDialogSize (hwndVPCDlg, hwndVPC);
                  }
               break;

            case IDM_VPC_COMP_SET:
               if (!wVdiscDevID)
                  {
                  DisplayMessage ("No videodisc device available");
                  break;
                  }

               /* Create a COMPLETE Video Player Control */

               /* If it is already minimal, just show it */
               if (hwndVPCDlg && (dwVPCShape & MCS_COMPLETE))
                  {
                  WinSetWindowPos (hwndVPCDlg, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
                  }
               else
                  {
                  if (hwndVPCDlg)
                     WinDestroyWindow(hwndVPCDlg);

                  dwVPCShape = MCS_COMPLETE;
                  hwndVPCDlg = WinLoadDlg(HWND_DESKTOP,
                                       hwndClient,
                                       pVPCDlgProc,
                                       0,
                                       ID_DLG_VPC,
                                       NULL);
                  OptimizeControlDialogSize (hwndVPCDlg, hwndVPC);
                  }
               break;

            case IDM_SHOWALL: /* SHOW ALL THE CONTROL DIALOGS */
               if (hwndAACDlg)
                  WinShowWindow (hwndAACDlg, TRUE);
               if (hwndVACDlg)
                  WinShowWindow (hwndVACDlg, TRUE);
               if (hwndVPCDlg)
                  WinShowWindow (hwndVPCDlg, TRUE);
               break;

            case IDM_SHOWAAC:
               if (hwndAACDlg)
                  WinShowWindow (hwndAACDlg, TRUE);
               break;

            case IDM_SHOWVAC:
               if (hwndVACDlg)
                  WinShowWindow (hwndVACDlg, TRUE);
               break;

            case IDM_SHOWVPC:
               if (hwndVPCDlg)
                  WinShowWindow (hwndVPCDlg, TRUE);
               break;

            case IDM_HIDEALL: /* HIDE ALL THE CONTROL DIALOGS */
               if (hwndAACDlg)
                  WinShowWindow (hwndAACDlg, FALSE);
               if (hwndVACDlg)
                  WinShowWindow (hwndVACDlg, FALSE);
               if (hwndVPCDlg)
                  WinShowWindow (hwndVPCDlg, FALSE);
               break;

            case IDM_HIDEAAC:
               if (hwndAACDlg)
                  WinShowWindow (hwndAACDlg, FALSE);
               break;

            case IDM_HIDEVAC:
               if (hwndVACDlg)
                  WinShowWindow (hwndVACDlg, FALSE);
               break;

            case IDM_HIDEVPC:
               if (hwndVPCDlg)
                  WinShowWindow (hwndVPCDlg, FALSE);
               break;

            /****************************************/
            /****     SOURCE SUBMENU OPTIONS     ****/
            /****************************************/
            case IDM_AUDIOSOURCE1:
               if (SelectSource (wAmpMixDevID, 2,
                                 MCI_LINE_IN_CONNECTOR, MCI_DISABLE_CONNECTOR))
                  DisplayMessage ("Could not change the audio source");
               if (SelectSource (wAmpMixDevID, 1,
                                 MCI_LINE_IN_CONNECTOR, MCI_ENABLE_CONNECTOR))
                  DisplayMessage ("Could not change the audio source");
               break;

            case IDM_AUDIOSOURCE2:
               if (SelectSource (wAmpMixDevID, 1,
                                 MCI_LINE_IN_CONNECTOR, MCI_DISABLE_CONNECTOR))
                  DisplayMessage ("Could not change the audio source");
               if (SelectSource (wAmpMixDevID, 2,
                                 MCI_LINE_IN_CONNECTOR, MCI_ENABLE_CONNECTOR))
                  DisplayMessage ("Could not change the audio source");
               break;

            case IDM_VIDEOSOURCE1:
               if (SelectSource (wOverlayDevID, 1,
                                 MCI_VIDEO_IN_CONNECTOR, MCI_ENABLE_CONNECTOR))
                  DisplayMessage ("Could not change the video source");
               break;

            case IDM_VIDEOSOURCE2:
               if (SelectSource (wOverlayDevID, 2,
                                 MCI_VIDEO_IN_CONNECTOR, MCI_ENABLE_CONNECTOR))
                  DisplayMessage ("Could not change the video source");
               break;

            case IDM_VIDEOSOURCE3:
               if (SelectSource (wOverlayDevID, 3,
                                 MCI_VIDEO_IN_CONNECTOR, MCI_ENABLE_CONNECTOR))
                  DisplayMessage ("Could not change the video source");
               break;

            /***************************************/
            /****     IMAGE SUBMENU OPTIONS     ****/
            /***************************************/

            case IDM_CAPTURE:
               /* Capture the ENTIRE area of the video window to
                  the image element buffer */
               if (!Capture (wOverlayDevID, 0, 0, 0, 0, 0))
                  /* there is now an element to use for save/restore */
                  fElement = TRUE;
               break;

            case IDM_RESTORE:
               /* Restore the current image element buffer to the
                  video window  */
               if (fElement)
                  Restore (wOverlayDevID, 0, 0, 0, 0, 0);
               break;

            case IDM_UNFREEZE:
               CheckUnFreeze (wOverlayDevID, 0, 0, 0, 0, 0);
               break;

            /**************************************/
            /****     HELP SUBMENU OPTIONS     ****/
            /**************************************/
            case IDM_HELPITEM:

                WinDlgBox(HWND_DESKTOP, hwnd,
                        VidplayHelpInfoDlgProc,
                        (HMODULE) 0, ID_HELP_INFO_DLG,
                        0);
               break;

            case IDM_ABOUT:

                  WinDlgBox( HWND_DESKTOP,
                        HWND_DESKTOP,
                        ( PFNWP )ProdInfoDlgProc,
                        ( HMODULE ) 0,
                        ID_DLG_PRODINFO,
                        NULL );
               break;

            //================================================================
            // MCI Device Activation/Acquisition Messages
            //================================================================

            default:
               return WinDefWindowProc( hwnd, msg, mp1, mp2 );
            }
         break;

      case MM_MCIPASSDEVICE:
         {
         USHORT   usPassCommand;              // WM_COMMAND command value
         WORD     wDevID;

         wDevID = SHORT1FROMMP (mp1);
         usPassCommand = SHORT1FROMMP (mp2);

         if (wDevID == wOverlayDevID)
            fOverlayAcquired = usPassCommand == MCI_GAINING_USE;
         if (wDevID == wVdiscDevID)
            fVdiscAcquired = usPassCommand == MCI_GAINING_USE;
         if (wDevID == wAmpMixDevID)
            fAmpMixAcquired = usPassCommand == MCI_GAINING_USE;

         // If one device is acquired, be sure that the other two are
         // acquired, too.
         if (usPassCommand == MCI_GAINING_USE)
            {
            // Overlay.
            if (wOverlayDevID && !fOverlayAcquired)
               AcquireDevice(wOverlayDevID, 0);

            // AmpMixer
            if (wAmpMixDevID && !fAmpMixAcquired)
               AcquireDevice(wAmpMixDevID, 0);

            // Videodisc player
            if (wVdiscDevID && !fVdiscAcquired)
               AcquireDevice(wVdiscDevID, 0);
            }

/* Include the following code if you wish to track the various */
/* device activations being lost to or gained from other instances */
#ifdef IFDESIRED
         switch (usPassCommand)
            {
            case MCI_LOSING_USE:
                DisplayMessage ("PASSDEVICE Losing Use");
                break;
            case MCI_GAINING_USE:
                DisplayMessage ("PASSDEVICE Gaining Use");
                break;
            default:
                DisplayMessage ("MM_MCIPASSDEVICE - unknown event");
                break;
            }
#endif
         }
         break;

      /****************************************/
      /**** Image File DragDrop Processing ****/
      /****************************************/
      case DM_DRAGOVER:    /* Determine if this object type is a FILE */
         {
         DRAGITEM Dragitem;

         /* Obtain access to the shared-memory area that describes
            the object being DRAGGED.  It has not yet been dropped */
         if (!DrgAccessDraginfo ((PDRAGINFO) mp1))
            return ((MRESULT)MPFROM2SHORT (DOR_NEVERDROP, DO_UNKNOWN));

         /* Obtain the first item in the list of items dragged */
         if (DrgQueryDragitem ((PDRAGINFO) mp1, sizeof (DRAGITEM),
                               &Dragitem, 0))
            {
            /* Check if the object is an OS/2 FILE.  That is the ONLY
               valid object type to drag onto VIDPLAY */
            if (!DrgVerifyRMF (&Dragitem, "DRM_OS2FILE", ""))
               {
               /* This IS an OS/2 file, so allow the drop */
               DrgFreeDraginfo ((PDRAGINFO) mp1);
               return ((MRESULT)MPFROM2SHORT (DOR_DROP, DO_UNKNOWN));
               }
            }

         DrgFreeDraginfo ((PDRAGINFO) mp1);
         return ((MRESULT)MPFROM2SHORT (DOR_NEVERDROP, DO_UNKNOWN));
         }

      case DM_DROP:             /* When dropped, get info about the object */
         {
         DRAGITEM Dragitem;
         CHAR szFileNameStr[256];
         CHAR szContainerNameStr[256];
         WORD wError = FALSE;

         /* Obtain the shared-memory memory describin the dropped object */
         if (!DrgAccessDraginfo ((PDRAGINFO) mp1))
            return (0);

         /* Obtain the first item in the list of items dragged */
         if (DrgQueryDragitem ((PDRAGINFO) mp1, sizeof (DRAGITEM),
                               &Dragitem, 0))
            {
            /* Obtain the container name - this SHOULD be the drive/dir */
            if (DrgQueryStrName (Dragitem.hstrContainerName,
                                 sizeof (szContainerNameStr), szContainerNameStr))
               strcpy (szTempElementName, szContainerNameStr);
            else
               {
               DisplayMessage ("No path name for dragged object");
               wError = TRUE;
               }

            /* Obtain the source name - this SHOULD be the file name */
            if (DrgQueryStrName (Dragitem.hstrSourceName,
                                 sizeof (szFileNameStr), szFileNameStr))
               strcat (szTempElementName, szFileNameStr);
            else
               {
               DisplayMessage ("No file name for dragged object");
               wError = TRUE;
               }
            }
         else
            {
            DisplayMessage ("No Drag item identified");
            wError = TRUE;
            }

         DrgFreeDraginfo ((PDRAGINFO) mp1);
         if (wError)
            return (0);

         /************************************************************/
         /* If all is OK so far, attempt to load the file.  The load */
         /* process will validate the file - it just won't load if   */
         /* it is not a support image format                         */
         /************************************************************/
         fElement = FALSE;
         if (LoadElement (wOverlayDevID, szTempElementName))
            DisplayMessage ("File requested could not be loaded");
         else
            {
            /* If the load was successful, then restore it to the screen */
            fElement = TRUE;
            Restore (wOverlayDevID, 0, 0, 0, 0, 0);
            }
         return;
         }

      /******************************************/
      /**** Screen Control/Painting Messages ****/
      /******************************************/
      case WM_ERASEBACKGROUND:
         // Paint the window background in the transparency color, if set
         if (TransColor)
            {
            WinFillRect((HPS)mp1, (PRECTL)mp2, TransColor);
            return (MRESULT)FALSE;
            }
         else
            // No transparency color set. Tell PM to paint the background
            return (MRESULT)TRUE;
         break;

      case WM_PAINT:
         {
         HPS    hps;                      /* Presentation space handle    */
         RECTL  rc;                       /* Window rectangle             */

         hps = WinBeginPaint( hwnd, (HPS)NULL, &rc );
         WinEndPaint( hps );               /* Drawing is complete          */
         }
         break;

      case WM_CLOSE:
         /* Close the Controls */
         if (hwndAACDlg)
            WinDestroyWindow(hwndAACDlg);
         if (hwndVACDlg)
            WinDestroyWindow(hwndVACDlg);
         if (hwndVPCDlg)
            WinDestroyWindow(hwndVPCDlg);

         /* Close the Devices */
         if (wAmpMixDevID)
            CloseDevice (wAmpMixDevID);
         if (wVdiscDevID)
            CloseDevice (wVdiscDevID);
         WinDestroyWindow(hwnd); // Avoid painting pink background on exit
         if (wOverlayDevID)
            CloseDevice (wOverlayDevID);

         WinPostMsg( hwndFrame, WM_QUIT, NULL, NULL);     /* Cause termination    */
         break;

      default:
         /* Must have WinDefWindowProc in your window procedure */
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }

/*********************** End of MainWndProc  *************************/

/*********************************************************************
 * Function : OptimizeControlDialogSize
 *
 * Description : This function will change the size of the dialog
 *               containing the multimedia control to be the optimal
 *               size for the control.
 *
 * Parameters :
 *      hwndControlDlg (INPUT) : Dialog (to be resized) holding the control
 *      hwndControl    (INPUT) : The control around which the dialog appears.
 *
 * Return Code :
 *      None
 *********************************************************************/
VOID OptimizeControlDialogSize (HWND hwndControlDlg, HWND hwndControl)
{
    SWP swp;
    SWP swpDlg;

    WinQueryWindowPos(hwndControl, &swp);
    WinQueryWindowPos(hwndControlDlg, &swpDlg);

    // Horizontal size -
    // Control offset is used as a margin (2x) plus the width of the control
    // plus the width of the borders around the dialog
    swpDlg.cx = 2 * (swp.x-1) + swp.cx + 2 * WinQuerySysValue(HWND_DESKTOP, SV_CXBORDER);

    // Vertical size -
    // Control offset is used as a margin (2x) plus the height of the control
    // plus the height of the border around the dialog, plus the height of
    // the dialog title bar
    swpDlg.cy = 2 * (swp.y-1) + swp.cy + 2 * WinQuerySysValue(HWND_DESKTOP, SV_CYBORDER) +
       WinQuerySysValue(HWND_DESKTOP, SV_CYTITLEBAR);

    WinSetWindowPos(hwndControlDlg, (HWND)0, 0, 0, swpDlg.cx, swpDlg.cy, SWP_SIZE | SWP_SHOW);
}

/*********************************************************************
 *  Name : ProdInfoDlgProc
 *
 *  Description : Processes all messages sent to the Product
 *                Information dialog box.
 *
 *  Concepts : Called for each message sent to the Product
 *             Information dialog box.  The Product
 *             Information box only has a button control so
 *             this routine only processes WM_COMMAND
 *             messages.  Any WM_COMMAND posted must have come
 *             from the Ok button so we dismiss the dialog
 *             upon receiving it.
 *
 * Parameters   : hwnd - Window handle to which message is addressed
 *                msg - Message type
 *                mp1 - First message parameter
 *                mp2 - Second message parameter
 *
 *  Returns : Dependent upon message sent
 *
 ****************************************************************/
MRESULT EXPENTRY ProdInfoDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
   {
   switch(msg)
      {
      case WM_INITDLG:
         break;

      case WM_COMMAND:
         /* No matter what the command, close the dialog */
         WinDismissDlg(hwnd, TRUE);
         break;

      default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
       }
   return (MRESULT)0;
   }
/********************** End of ProdInfoDlgProc *********************/

/****************************************************************
 *  Name:   VidplayHelpInfoDlgProc
 *
 *  Description : This dialog procedure handles the dialog help
 *                window for VIDPLAY. It displays a window with
 *                a list box where the help information text is
 *                displayed. The list box is scrollable. The dialog
 *                box has a system menu only. Thus, the user must
 *                close it from the system menu.
 *
 *  Concepts : List Box with help information.
 *
 *  Parameters   : hwnd - Window handle to which message is addressed
 *                 msg - Message type
 *                 mp1 - First message parameter
 *                 mp2 - Second message parameter
 *
 *  Returns:  Return values are determined by each message
 *
 ****************************************************************/
MRESULT EXPENTRY VidplayHelpInfoDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {
   int count;
   COLOR   TransColor = CLR_PALEGRAY;                  /* Window Color */
   CHAR szTempStr[100];

   switch ( msg )
      {
      case WM_INITDLG:
         /* Create help dialog and insert the lines of text */
         for ( count=0 ; count<NUM_HELP_LINES ; count++ )
            {
            WinLoadString (hab,
                           (HMODULE) 0,
                           (ULONG)ID_HELP_TEXT_LINE_1 + count,
                           (LONG)100,
                           szTempStr);

            WinSendDlgItemMsg( hwndDlg,
                              ID_HELP_LISTBOX,
                              LM_INSERTITEM,
                              MPFROMSHORT(LIT_END),
                              MPFROMP (szTempStr));

            WinSetPresParam (
               hwndDlg,
               PP_BACKGROUNDCOLORINDEX,
               (ULONG) sizeof (TransColor),
               (PVOID) &TransColor);

            } /* endfor */
         break;

      default:
          /******************************************************************/
          /* Any event messages that the dialog procedure has not processed */
          /* come here and are processed by WinDefDlgProc.                  */
          /* This call MUST exist in your dialog procedure.                 */
          /******************************************************************/
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }

/****************************************************************
 *  Name:   pAACDlgProc
 *
 *  Description :
 *      Places a Audio Attributes Control on the dialog, and attaches the
 *      Audio Attributes Control to the Amp/Mixer device previously opened.
 *
 *  Parameters :
 *      Standard Dialog Proc parameters
 *
 *  Returns:
 *      Standard Dialog Proc return codes
 *
 ****************************************************************/
MRESULT EXPENTRY pAACDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {

   switch ( msg )
      {
      /*********************************************************************
       * The following code is optional, and illustrates the processing of *
       * WM_CONTROL messages for the Audio Attributes Control              *
       *********************************************************************/
#ifdef OPTIONAL
      case WM_CONTROL:
         /* A child-control on the MM Control was manipulated */
         if (LOUSHORT (mp1) == ID_AAC_WND)
            {
            switch (HIUSHORT (mp1))
               {
               case ID_VOLUME:
                  DisplayMessage ("Volume dial used");
                  break;

               case ID_BALANCE:
               case ID_TREBLE:
               // And others...
                  break;

               case MCN_ERROR:
                  switch (LOUSHORT (mp2))
                     {
                     case ID_VOLUME:
                         DisplayMessage ("Volume dial error");
                         break;
                     }

                  /* Display error from MMPM driver */
                  if (HIUSHORT (mp2) > MCIERR_BASE)
                     {
                     CHAR   caErrString[128];

                     memset(caErrString, '\0', sizeof(caErrString));
                     mciGetErrorString((DWORD) HIUSHORT(mp2),
                                       (LPSTR) caErrString,
                                       (WORD)  sizeof(caErrString) );

                     DisplayMessage (caErrString);
                     }
                  else
                     DisplayMessage ("Some other error");
                  break;
               } /* endswitch */
            }
        break;
#endif
      case WM_INITDLG:
         {
         /* Create an Audio Attributes Control */
         hwndAAC = WinCreateWindow
                      (hwndDlg,
                      WC_AUDIOCTL,
                      "",
                      WS_VISIBLE | MCS_VERTICAL | dwAACShape,
                      X_MMCTL, Y_MMCTL,
                      200, 100,
                      hwndDlg,
                      HWND_TOP,
                      ID_AAC_WND,
                      NULL,
                      NULL);

         /* if the control was created, attach it to the Amp/Mixer device */
         if (hwndAAC)
            {
            if (!WinSendMsg (hwndAAC, MCM_SETDEVICEALIAS, szAmpMixAlias, 0))
               DisplayMessage ("Cannot attach audio attributes control to ampmixer device");
            }
         else
            DisplayMessage ("Cannot create Audio Attributes Control");
         }
         break;

      case WM_ACTIVATE:
         /* ensure that we have acquired the device whenever this
            window is accessed by the user */
         AcquireDevice (wAmpMixDevID, 0);
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );


      case WM_DESTROY:
         hwndAACDlg = 0;
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );

      default:
          /******************************************************************/
          /* Any event messages that the dialog procedure has not processed */
          /* come here and are processed by WinDefDlgProc.                  */
          /* This call MUST exist in your dialog procedure.                 */
          /******************************************************************/
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }
/****************************************************************
 *  Name:   pVACDlgProc
 *
 *  Description :
 *      Places a Video Attributes Control on the dialog, and attaches the
 *      Video Attributes Control to the Overlay device previously opened.
 *
 *  Parameters :
 *      Standard Dialog Proc parameters
 *
 *  Returns:
 *      Standard Dialog Proc return codes
 *
 ****************************************************************/
MRESULT EXPENTRY pVACDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {

   switch ( msg )
      {
      /*********************************************************************
       * The following code is optional, and illustrates the processing of *
       * WM_CONTROL messages for the Audio Attributes Control              *
       *********************************************************************/
#ifdef OPTIONAL
      case WM_CONTROL:
         /* A child-control on the MM Control was manipulated */
         if (LOUSHORT (mp1) == ID_VAC_WND)
            {
            switch (HIUSHORT (mp1))
               {
               case ID_BRIGHTNESS:
                  DisplayMessage ("Brightness dial used");
                  break;

               case ID_HUE:
               case ID_CONTRAST:
               // And others...
                  break;

               case MCN_ERROR:
                  switch (LOUSHORT (mp2))
                     {
                     case ID_BRIGHTNESS:
                         DisplayMessage ("Brightness dial error");
                         break;
                     }

                  /* Display error from MMPM driver */
                  if (HIUSHORT (mp2) > MCIERR_BASE)
                     {
                     CHAR   caErrString[128];

                     memset(caErrString, '\0', sizeof(caErrString));
                     mciGetErrorString((DWORD) HIUSHORT(mp2),
                                       (LPSTR) caErrString,
                                       (WORD)  sizeof(caErrString) );

                     DisplayMessage (caErrString);
                     }
                  else
                     DisplayMessage ("Some other error");
                  break;
               } /* endswitch */
            }
        break;
#endif

      case WM_INITDLG:
         {
         /* Create a Video Attributes Control */
         hwndVAC = WinCreateWindow
                      (hwndDlg,
                      WC_VIDEOCTL,
                      "",
                      WS_VISIBLE | MCS_VERTICAL | dwVACShape,
                      X_MMCTL, Y_MMCTL,
                      200, 100,
                      hwndDlg,
                      HWND_TOP,
                      ID_VAC_WND,
                      NULL,
                      NULL);

         /* if the control was created, attach it to the Overlay device */
         if (hwndVAC)
            {
            if (!WinSendMsg (hwndVAC, MCM_SETDEVICEALIAS, szOverlayAlias, 0))
               DisplayMessage ("Cannot attach video attributes control to overlay device");
            }
         else
            DisplayMessage ("Cannot create Video Attributes Control");
         }
         break;

      case WM_ACTIVATE:
         /* ensure that we have acquired the device whenever this
            window is accessed by the user */
         if (!fOverlayAcquired)
            {
            WinSetWindowPos (hwndFrame, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER | SWP_SHOW);
            AcquireDevice (wOverlayDevID, 0);
            }
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );

      case WM_DESTROY:
         hwndVACDlg = 0;
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );

      default:
          /******************************************************************/
          /* Any event messages that the dialog procedure has not processed */
          /* come here and are processed by WinDefDlgProc.                  */
          /* This call MUST exist in your dialog procedure.                 */
          /******************************************************************/
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }
/****************************************************************
 *  Name:   pVPCDlgProc
 *
 *  Description :
 *      Places a Player Control on the dialog, and attaches the Player
 *      Control to the videodisc player device previously opened.
 *
 *  Parameters :
 *      Standard Dialog Proc parameters
 *
 *  Returns:
 *      Standard Dialog Proc return codes
 *
 ****************************************************************/
MRESULT EXPENTRY pVPCDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {

   switch ( msg )
      {
      /*********************************************************************
       * The following code is optional, and illustrates the processing of *
       * WM_CONTROL messages for the Player Control.                       *
       *********************************************************************/
#ifdef OPTIONAL
      case WM_CONTROL:
         /* A child-control on the MM Control was manipulated */
         if (LOUSHORT (mp1) == ID_VPC_WND)
            {
            switch (HIUSHORT (mp1))
               {
               case ID_PLAY:
                  DisplayMessage ("Play button pressed");
                  break;

               case ID_PLAYB:
               case ID_SCANB:
               case ID_SCANF:
               // And others...
                  break;

               case MCN_ERROR:
                  switch (LOUSHORT (mp2))
                     {
                     case ID_PLAY:
                         DisplayMessage ("Play button error");
                         break;
                     }

                  /* Display error from MMPM driver */
                  if (HIUSHORT (mp2) > MCIERR_BASE)
                     {
                     CHAR   caErrString[128];

                     memset(caErrString, '\0', sizeof(caErrString));
                     mciGetErrorString((DWORD) HIUSHORT(mp2),
                                       (LPSTR) caErrString,
                                       (WORD)  sizeof(caErrString) );

                     DisplayMessage (caErrString);
                     }
                  else
                     DisplayMessage ("Some other error");
                  break;
               } /* endswitch */
            }

         /* An error occurred in MM Control processing */
         break;
#endif

      case WM_INITDLG:
         {
         /* Create a Player Control */
         hwndVPC = WinCreateWindow
                      (hwndDlg,
                      WC_PLAYERCTL,
                      "",
                      WS_VISIBLE | dwVPCShape,
                      X_MMCTL, Y_MMCTL,
                      200, 100,
                      hwndDlg,
                      HWND_TOP,
                      ID_VPC_WND,
                      NULL,
                      NULL);

         /* if the control was created, attach it to the Videodisc device */
         if (hwndVPC)
            {
            if (!WinSendMsg (hwndVPC, MCM_SETDEVICEALIAS, szVdiscAlias, 0))
               DisplayMessage ("Cannot attach videodisc control to player");
            }
         else
            DisplayMessage ("Cannot create Videodisc Control");
         }
         break;

      case WM_ACTIVATE:
         /* ensure that we have acquired the device whenever this
            window is accessed by the user */
         AcquireDevice (wVdiscDevID, 0);
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );

      case WM_DESTROY:
         hwndVPCDlg = 0;
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );

      default:
          /******************************************************************/
          /* Any event messages that the dialog procedure has not processed */
          /* come here and are processed by WinDefDlgProc.                  */
          /* This call MUST exist in your dialog procedure.                 */
          /******************************************************************/
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }


/******************** End of VidplayHelpUnfoDlgProc **********************/

/*********************************************************************/
/*********************************************************************/
/***************    Miscellaneous Support Functions    ***************/
/*********************************************************************/
/*********************************************************************/
/****************************************************************
 * Function : GetElementName
 *
 * Description :
 *      This function will display the Standard File Dialog
 *      to the user, by which the user can select a file name.
 *      This file name will be returned to the calling function.
 *
 *      This function is used for both the Load and SaveAs
 *      menu selection.
 *
 * Concepts :
 *      Normal PM programming using the Standard File Dialog
 *
 * Parameters :
 *      hwnd    : Name of the parent/owner for the dialog being created
 *      pszTitle (input)        : String title to be displayed -
 *                                different for Load, Save, etc
 *      pszElementName (output) : File selected by the user
 *
 * Returns :
 *
 ****************************************************************/
DWORD GetElementName (HWND hwnd, PSZ pszTitle, PSZ pszElementName)
   {
   FILEDLG pfdFiledlg;      /* File dialog info structure           */
   char pszFullFile[CCHMAXPATH] = "*.*";/* File filter string       */
   HWND hwndDlg;            /* File dialog window                   */

   /*****************************************************************/
   /* Initially set all fields to 0                                 */
   /*****************************************************************/

   memset(&pfdFiledlg, 0, sizeof(FILEDLG));

   /*****************************************************************/
   /* Initialize those fields in the FILEDLG structure that are     */
   /* used by the application                                       */
   /*****************************************************************/
   pfdFiledlg.cbSize = sizeof(FILEDLG); /* Size of structure        */
   pfdFiledlg.fl = FDS_HELPBUTTON | FDS_CENTER | FDS_OPEN_DIALOG;
                                        /* FDS_* flags              */
   pfdFiledlg.pszTitle = pszTitle;      /* Dialog title string      */
   strcpy(pfdFiledlg.szFullFile, pszFullFile); /* Initial path,     */
                                               /* file name, or     */
                                               /* file filter       */

   /*****************************************************************/
   /* Display the dialog and get the file                           */
   /*****************************************************************/

   hwndDlg = WinFileDlg(HWND_DESKTOP, hwnd, &pfdFiledlg);

   if (hwndDlg && (pfdFiledlg.lReturn == DID_OK))
      {
      /**************************************************************/
      /* Upon successful return of a file, open it for reading and  */
      /* further processing                                         */
      /**************************************************************/

      if (pszElementName)
         strcpy (pszElementName, pfdFiledlg.szFullFile);
      return (FALSE);
      }

   return (TRUE);
   }


/****************************************************************
 *  Name:   DisplayMessage
 *
 *  Description :
 *      Displays a message box containing an error/information
 *      message each time called.
 *
 *  Parameters   : str - error message to display
 *
 *  Returns:  Void
 *
 ****************************************************************/
VOID DisplayMessage(PCH str)
   {
   /********************************************************************/
   /* This routine calls WinMessageBox to dsplay error messages to the */
   /* user.                                                            */
   /********************************************************************/

   WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PCH) str,
      (PCH) "MessageBox For the VIDPLAY Sample", ID_MESSAGEBOX,
      MB_OK | MB_APPLMODAL | MB_MOVEABLE );
   }
/******************* End of DisplayMessage procedure ********************/



/**********************************************************************/
/**********************************************************************/
/**********************************************************************/
/*    MCI Calls Section                                               */
/**********************************************************************/
/**********************************************************************/
/**********************************************************************/

/**********************************************************************
 * Function : OpenDevice
 *
 * Description :
 *      Instructs MMPM to attempt to connect this application to the
 *      desired device.  If successful, the device will allocate
 *      resources and perform other initialization functions.
 *
 * Parameters :
 *      lpstrDevName (input) : MMPM install name of the device, such
 *                             as videodisc, overlay, etc.
 *      lpstrDevAlias (input): string name by which the device can also
 *                             be controlled.  This can be anything the
 *                             application desires.  It is required if
 *                             the multimedia controls are to be used.
 *      pwDeviceID  (output) : MMPM ID of the device opened, if successful
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 *
 **********************************************************************/
DWORD OpenDevice (LPSTR lpstrDevName, LPSTR lpstrDevAlias, WORD *pwDeviceID)
   {
   MCI_VID_OPEN_PARMS  mciOpenParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   mciOpenParms.dwCallback = (DWORD)hwndNotify;
   mciOpenParms.wDeviceID = 0;
   mciOpenParms.lpstrDeviceType = lpstrDevName;
   mciOpenParms.lpstrElementName = 0;

   mciOpenParms.lpstrAlias = lpstrDevAlias;
   mciOpenParms.hwndParent = 0;

   dwReturn = mciSendCommand(0, MCI_OPEN,
                  MCI_WAIT | MCI_OPEN_SHAREABLE | MCI_OPEN_ALIAS,
                  (DWORD)&mciOpenParms,
                  wUserParm);

   if (LOUSHORT(dwReturn) == MCIERR_SUCCESS)
      {
      *pwDeviceID = mciOpenParms.wDeviceID;
      return (FALSE);
      }

   return (TRUE);
   }

/**********************************************************************
 * Function : CloseDevice
 *
 * Description :
 *      Instructs the device to shut down, release all resources, and
 *      for MMPM to disconnect this application from the device.
 *
 * Parameters :
 *      wDeviceID (input)   : the device to be closed
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 *
 ***********************************************************************/
DWORD CloseDevice (WORD wDeviceID)
   {
   MCI_GENERIC_PARMS  mciGenericParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   mciGenericParms.dwCallback = (DWORD)hwndNotify;

   dwReturn = mciSendCommand(wDeviceID, MCI_CLOSE,
                  MCI_WAIT,
                  (DWORD)&mciGenericParms,
                  wUserParm);

   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }


/**********************************************************************
 * Function : AcquireDevice
 *
 * Description :
 *      Instructs MMPM to acquire this device for use.  It may have
 *      been lost to some other application.  No device operations
 *      can be performed until the device control is re-acquired.
 *
 * Parameters :
 *      wDeviceID (input)   : the device to acquire
 *      dwExclusive (input) : indicates if the device is to be
 *                            acquired shareable or exclusively.
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 **********************************************************************/
DWORD AcquireDevice (WORD wDeviceID, DWORD dwExclusive)
   {
   MCI_GENERIC_PARMS  mciGenericParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   mciGenericParms.dwCallback = (DWORD)hwndNotify;
   dwReturn = mciSendCommand(wDeviceID, MCI_ACQUIREDEVICE,
                  MCI_WAIT | dwExclusive,
                  (DWORD)&mciGenericParms,
                  wUserParm);

   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }

#ifdef IFNEEDED
/**********************************************************************
 * Function : ReleaseDevice
 *
 * Description :
 *      Instructs
 *
 * Parameters :
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 *
 **********************************************************************/
DWORD ReleaseDevice (WORD wDeviceID)
   {
   MCI_GENERIC_PARMS  mciGenericParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   mciGenericParms.dwCallback = (DWORD)hwndNotify;
   dwReturn = mciSendCommand(wDeviceID, MCI_RELEASEDEVICE,
                  MCI_WAIT,
                  (DWORD)&mciGenericParms,
                  wUserParm);

   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }
#endif

/**********************************************************************
 * Function : LoadElement
 *
 * Description :
 *      Instructs the overlay device to attempt to load the file
 *      specified into the image element buffer.  This image will
 *      be loaded offscreen.  It can then be saved or restored to
 *      the screen.
 *
 * Parameters :
 *      wDeviceID (input)   : the Overlay device
 *      pszName (input)     : Name of file to load
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 **********************************************************************/
DWORD LoadElement (WORD wDeviceID, PSZ pszName)
   {
   MCI_LOAD_PARMS  mciLoadParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   mciLoadParms.dwCallback = (DWORD)hwndNotify;
   mciLoadParms.lpstrElementName = (LPSTR)pszName;

   dwReturn = mciSendCommand(wDeviceID, MCI_LOAD,
                  MCI_WAIT | MCI_OPEN_ELEMENT,
                  (DWORD)&mciLoadParms,
                  wUserParm);


   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }

/**********************************************************************
 * Function : SaveElement
 *
 * Description :
 *      Instructs the overlay device to save the current image element
 *      buffer to a file.  A specific file can be indicated.  If no
 *      specific file is indicated, the save will use the last file
 *      saved or loaded.
 *
 * Parameters :
 *      wDeviceID (input)   : the Overlay device
 *      dwFlag (input)      : 0 or MCI_SAVE_FILE
 *      pszName (input)     : Name to use if MCI_SAVE_FILE is specified
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 *
 **********************************************************************/
DWORD SaveElement (WORD wDeviceID, DWORD dwFlag, PSZ pszName)
   {
   MCI_SAVE_PARMS  mciSaveParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   mciSaveParms.dwCallback = (DWORD)hwndNotify;

   if (dwFlag & MCI_SAVE_FILE)
      mciSaveParms.lpFileName = (DWORD)pszName;
   else
      mciSaveParms.lpFileName = (DWORD)0;

   dwReturn = mciSendCommand(wDeviceID, MCI_SAVE,
                  MCI_WAIT | dwFlag,
                  (DWORD)&mciSaveParms,
                  wUserParm);

   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }

/**********************************************************************
 * Function : Capture
 *
 * Description :
 *      This function will instruct the overlay device to capture the
 *      video in the video window to the image element buffer.
 *      It will capture only that video in the sub-rectangle if requested,
 *
 * Parameters :
 *      wDeviceID (input)  : the Overlay device
 *      dwFlag  (input)    : 0 or MCI_CAPTURE_RECT
 *      dwX1, dwY1,
 *      dwX2, dwY2 (input) : Sub-rectangle from which image is to be
 *                           captured if MCI_CAPTURE_RECT is specified
 *
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 **********************************************************************/
DWORD Capture (WORD wDeviceID, DWORD dwFlag,
               DWORD dwX1, DWORD dwY1,
               DWORD dwX2, DWORD dwY2)
   {
   MCI_CAPTURE_PARMS mciCaptureParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   memset (&mciCaptureParms, 0x00, sizeof (MCI_CAPTURE_PARMS));
   mciCaptureParms.dwCallback = (DWORD)hwndNotify;
   mciCaptureParms.rect.xLeft   = dwX1;
   mciCaptureParms.rect.yBottom = dwY1;
   mciCaptureParms.rect.xRight  = dwX2;
   mciCaptureParms.rect.yTop    = dwY2;


   dwReturn = mciSendCommand(wDeviceID, MCI_CAPTURE,
                  MCI_WAIT | dwFlag,
                  (DWORD)&mciCaptureParms,
                  wUserParm);

   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }

/**********************************************************************
 * Function : Restore
 *
 * Description :
 *      This function will instruct the overlay device to restore the
 *      current image element to the video window.  It will place the
 *      image in the sub-rectangle if requested, where the lower-left
 *      corner of the image appears in the lower-left corner of the
 *      sub-rectangle.
 *
 * Parameters :
 *      wDeviceID (input)  : the Overlay device
 *      dwFlag  (input)    : 0 or MCI_RESTORE_DEST_RECT
 *      dwX1, dwY1,
 *      dwX2, dwY2 (input) : Sub-rectangle in which image is to be
 *                           restored if MCI_RESTORE_DEST_RECT is
 *                           specified.
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 ***********************************************************************/
DWORD Restore (WORD wDeviceID, DWORD dwFlag,
                   DWORD dwX1, DWORD dwY1,
                   DWORD dwX2, DWORD dwY2)
   {
   MCI_RESTORE_PARMS mciRestoreParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   memset (&mciRestoreParms, 0x00, sizeof (MCI_RESTORE_PARMS));
   mciRestoreParms.dwCallback = (DWORD)hwndNotify;
   mciRestoreParms.DestRect.xLeft   = dwX1;
   mciRestoreParms.DestRect.yBottom = dwY1;
   mciRestoreParms.DestRect.xRight  = dwX2;
   mciRestoreParms.DestRect.yTop    = dwY2;


   dwReturn = mciSendCommand(wDeviceID, MCI_RESTORE,
                  MCI_WAIT | dwFlag,
                  (DWORD)&mciRestoreParms,
                  wUserParm);

   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }


/**********************************************************************
 * Function : CheckUnFreeze
 *
 * Description:
 *      Instructs the overlay device to unfreeze the entire window.
 *      This is necessary after a restore operation, which freezes
 *      the window so that the image will not be replaced by video.
 *
 * Parameters:
 *      wDeviceID (input)  : the Overlay device
 *      dwFlag (input)
 *              Freeze flags - indicating rectangle or freeze outside
 *      dwX1, dwY1, dwX2, dwY2 (input)
 *              Rectangle corners, if partial freeze was used
 *
 * Return Code:
 *      0 if all OK
 *      > 0 if error
 **********************************************************************/
DWORD CheckUnFreeze (WORD wDeviceID, DWORD dwFlag,
                     DWORD dwX1, DWORD dwY1, DWORD dwX2, DWORD dwY2)
   {
   MCI_VID_RECT_PARMS mciUnFreezeParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   memset (&mciUnFreezeParms, 0x00, sizeof (MCI_VID_RECT_PARMS));
   mciUnFreezeParms.dwCallback = (DWORD)hwndNotify;
   mciUnFreezeParms.rc.xLeft   = dwX1;
   mciUnFreezeParms.rc.yBottom = dwY1;
   mciUnFreezeParms.rc.xRight  = dwX2;
   mciUnFreezeParms.rc.yTop    = dwY2;


   dwReturn = mciSendCommand(wDeviceID, MCI_UNFREEZE,
                  dwInformType | dwFlag,
                  (DWORD)&mciUnFreezeParms,
                  wUserParm);

   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }


/************************************************************************
 * Function : UseAlternateWindow
 *
 * Description :
 *      Assigns an application window to be used as the video
 *      window by an Overlay device.
 *
 * Parameters :
 *      wDeviceID (input)     : the Overlay device
 *      hwndAlternate (input) : the window in which video is to be displayed
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 ************************************************************************/
DWORD  UseAlternateWindow (WORD wDeviceID, HWND hwndAlternate)
   {
   WORD    wUserParm = 0;
   DWORD   dwReturn;
   MCI_VID_WINDOW_PARMS mciWindowParms;

   memset (&mciWindowParms, 0x00, sizeof (MCI_VID_WINDOW_PARMS));
   mciWindowParms.dwCallback = (DWORD)0;
   mciWindowParms.hwndDest = hwndAlternate;

   dwReturn = mciSendCommand(wDeviceID, MCI_WINDOW,
                  MCI_WAIT | MCI_OVLY_WINDOW_HWND,
                  (DWORD)&mciWindowParms,
                  wUserParm);


   return (dwReturn);
   }

/************************************************************************
 * Function : CheckTransparencyColor
 *
 * Description :
 *      This function will query the overlay device to obtain
 *      the transparency color used by the device.  Video will
 *      display through all pels in the video window that are
 *      the transparency color.
 *
 * Parameters :
 *      wDeviceID (input)     : the Overlay device
 *      TransColor (output) : the transparency color used by the device
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 ************************************************************************/
DWORD CheckTransparencyColor (WORD wDeviceID, COLOR *TransColor)
   {
   WORD    wUserParm = 0;
   DWORD   dwReturn;
   MCI_STATUS_PARMS mciStatusParms;

   memset (&mciStatusParms, 0x00, sizeof (MCI_STATUS_PARMS));
   mciStatusParms.dwCallback = (DWORD)hwndNotify;
   mciStatusParms.dwItem = (DWORD)MCI_OVLY_STATUS_TRANSPARENT_COLOR;

   dwReturn = mciSendCommand(wDeviceID, MCI_STATUS,
                  dwInformType | MCI_STATUS_ITEM,
                  (DWORD)&mciStatusParms,
                  wUserParm);

   if (LOUSHORT(dwReturn) == MCIERR_SUCCESS)
      {
      *TransColor = (COLOR)mciStatusParms.dwReturn;
      return (MCIERR_SUCCESS);
      }

   return (TRUE);
   }

/**********************************************************************
 * Function : SelectSource
 *
 * Description :
 *      Instructs the device to uses the specified source for input,
 *      whether this is audio on the Amp/Mixer or video on the Overlay
 *      device.
 *
 * Parameters :
 *      wDeviceID (input)       : the Overlay or Amp/Mixer device
 *      dwSourceNum (input)     : the number of the source to be used.
 *      dwSourceType (input)    : the type of source
 *                                MCI_VIDEO_IN_CONNECTOR for video
 *                                MCI_LINE_IN_CONNECTOR for audio
 *      dwSetting (input)       : MCI_ENABLE_CONNECTOR or MCI_DISABLE_CONNECTOR
 *
 * Returns :
 *      0 if all OK
 *      > 0 if error
 *
 **********************************************************************/
DWORD SelectSource (WORD wDeviceID, DWORD dwSourceNum,
                    DWORD dwSourceType, DWORD dwSetting)
   {
   MCI_CONNECTOR_PARMS  mciConnectorParms;
   WORD    wUserParm = 0;
   DWORD   dwReturn;

   mciConnectorParms.dwCallback = (DWORD)hwndNotify;
   mciConnectorParms.dwConnectorType = dwSourceType;
   mciConnectorParms.dwConnectorIndex = dwSourceNum;


   dwReturn = mciSendCommand(wDeviceID, MCI_CONNECTOR,
                  MCI_WAIT | dwSetting |
                  MCI_CONNECTOR_TYPE | MCI_CONNECTOR_INDEX,
                  (DWORD)&mciConnectorParms,
                  wUserParm);

   return (LOUSHORT(dwReturn) != MCIERR_SUCCESS);
   }


/******************** End of DevHelpUnfoDlgProc **********************/
