//*****************************************************************************
//
// File Name: Vidview2.c
//
// ****!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*****
// ****************************************************************************
// *+*+* NOTE: This file contains both '//' style comments and "/*  */"   *+*+*
// **+**       style comments.  You will have to set the the /Ss+ compile *****
// *+*+*       option for this file to be compiled.                       *+*+*
// ****************************************************************************
// ****!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*****
//
// Description : VIDVIEW sample application source file.
//
// Copyright (C) 1992 IBM Corporation
//
//     DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
//     sample code created by IBM Corporation. This sample code is not
//     part of any standard or IBM product and is provided to you solely
//     for  the purpose of assisting you in the development of your
//     applications.  The code is provided "AS IS", without
//     warranty of any kind.  IBM shall not be liable for any damages
//     arising out of your use of the sample code, even if they have been
//     advised of the possibility of such damages.                                                    *
//
//
// A. Note to Programmers
//
//    This file contains functions that manage calculations when the
//    video source rectangle is allowed to vary continuously, and
//    when the video destination is allowed to be various shapes and
//    sizes.  This is done so as to allow the user to experiment
//    with different source-to-destination shapes and sizes.
//
//    In contrast, in a real-life video application, each video window
//    will probably have a fixed size (perhaps each video window will
//    be the same size, or perhaps there will be only 1 video window)
//    and the source rectangle will probably also be of a fixed size
//    (perhaps 688 by 480 or some number close to it).
//
//
// B. Primitive Functions for Overlay and Videodisc Player Devices
//
//    VIDVIEW.C and VIDVIEW2.C include primitive functions to
//    do overlay and videodisc operations.
//
//    These are listed in subgroups in VIDVIEW.H.
//
//*****************************************************************************

#define INCL_WIN
#define INCL_WINSYS
#define INCL_WINFRAMEMGR
#define INCL_WINMENUS
#define INCL_WINDIALOGS
#define INCL_WINSTDDLGS
#define INCL_DOSMODULEMGR
#define INCL_GPI
#define INCL_CIRCULARSLIDER
#define INCL_MMCTL
#define INCL_GPICONTROL

#include <os2.h>
#include <os2me.h>
#include <sw.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "lvdsys.h"
#include "mmctl.h"
#include "vidview.h"


//-----------------------------------------------------------------------------
// Externs and initializations...
//-----------------------------------------------------------------------------
extern PAPPLICATIONINFO   paiMain ;

LONG  aStretchWindowIDs [] =
   {
   ID_SRC_RCL_STRETCH_HORZ ,
   ID_SRC_RCL_STRETCH_VERT ,
   0
   } ;
LONG  aZoomWindowIDs [] =
   {
   ID_SRC_RCL_ZOOM_CONTROL,
   0,
   0
   } ;

SZ szFullScreenMessage [] =
   {
   "When the client area is screen-sized, the title bar lies 'above' "
   "the screen.\r"
   "You can move the application window down by using the Presentation "
   "Manager control keys (click with mouse button 1 on the client area, "
   "then press Alt-F7, then cursor down until the sizing rectangle appears, "
   "and finally press Enter).\r"
   "You can use the Alt-Esc to move the source control window back to the top."
   } ;

// Macros...
// Note that these 2 use the same conversion formula in BOTH...
//
// We want the vertical scroll bar to act like the higher numbers are at
// the TOP instead of the bottom; so we use these macros to switch
// between source buffer values and vertical scroll bar valuess.
#define VVM_SOURCE_TO_VSCROLL(length, thumbsize, thumbpos) (((length) - (thumbsize)) - (thumbpos))
#define VVM_VSCROLL_TO_SOURCE(length, thumbsize, thumbpos) ((((length) - (thumbsize)) - 1 ) - (thumbpos))

//----------------------------------------------------------------------------
// Name:          SetDefaultSourceRect
//
// Description:   Sets VIDVIEW's default source rectangle...
//
// Concepts:      Our default source rectangle is a 688 by 480 (NTSC)
//                or 688 by 548 (PAL) rectangle which is slightly
//                offset in the "video source buffer".  The
//                "video source buffer" is a concept which imagines that
//                the incoming video is arranged as a flat 706 by 484 buffer
//                in NTSC or 706 by 564 in PAL.
//
//                We can choose any rectangle within this array as our
//                source rectangle.  Then this source rectangle will be
//                projected onto the destination rectangle.
//
//                It is recommended to choose a source rectangle first,
//                because that determines how much of the incoming video image
//                you want to show.  Then choose a height or width for the
//                destination rectangle so that the image is easily
//                viewable.  Then calculate the other dimension
//                of the destination rectangle so that the destination
//                rectangle is proportional (same width-to-height ratio)
//                to the source rectangle.
//
//                Example: Source rectangle is 688 by 480.  We have an
//                app where the destination rectangle needs to be pretty big,
//                let's say 344 wide.  So destination rectangle height is
//
//                   PropHeight = ( PropWidth * AspectY ) / AspectX ;
//
//                Note that we multiply BEFORE the divide.
//
//                Also we will get better results if we round.  So
//                we change the equation to read:
//
//                   PropHeight = ( ( PropWidth * AspectY ) + ( AspectX / 2 ) )
//                                 / AspectX ;
//
//                Use the actual source rectangle height and width for "AspectY"
//                and "AspectX".  So we plug in the numbers and get:
//
//                   PropHeight = ( ( 344 * 480 ) + ( 688 / 2 ) ) / 688
//
//                              = 240
//
//
// Parameters:    pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID  SetDefaultSourceRect ( PVIDEOWINDOWINFO pvi )
   {
   // Set to default values...
   pvi->rclCurSrc = pvi->prclSrcRects
                       [ IDM_VDW_CLIENT_FULL_EXTENT - IDM_VDW_CHILD ] ;

   // Set source aspect ratio values from the original dimensions of the source...
   pvi->SrcAspectX = ( pvi->rclCurSrc.xRight  - pvi->rclCurSrc.xLeft   ) + 1 ;
   pvi->SrcAspectY = ( pvi->rclCurSrc.yTop    - pvi->rclCurSrc.yBottom ) + 1 ;

   // Set the overlay device to this source...
   OvlPutSrcRect ( paiMain->wOverlayDeviceID, &pvi->rclCurSrc, ( HWND ) 0 ) ;
   }


//----------------------------------------------------------------------------
// Name:          MakeSrcPropToNewDst()
//
// Description:   Generates a source rectangle for VIDVIEW proportional
//                (i.e. same width-to-height ratio) as new destination style...
//
// Concepts:      Usu we choose the source rectangle first; but here we
//                have allowed the user to change the destination rectangle,
//                so we have to scramble around and find a source rectangle
//                which is (1) the same aspect ratio as the new destination
//                rectangle, (2) at least as big as the new destination
//                rectangle.
//
// Parameters:    pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
//                OldStyle, NewStyle - "style" identifier of dst rect
//
// Returns:       TRUE = success.
//
//----------------------------------------------------------------------------
BOOL MakeSrcPropToNewDst ( PVIDEOWINDOWINFO pvi, LONG OldStyle, LONG NewStyle )
   {
   PRECTL prclSrc, prclDst, prclMinSrc ;
   LONG   NewSrcCode ;
   LONG   SWidth, SHeight, DWidth, DHeight ;


   // Init....
   prclSrc = &pvi->rclCurSrc  ;
   prclDst = &pvi->rclClient  ;

   // Switch on the new destination code...
   // ...If the new destination code has different proportions from
   // the old destination code, then indicate that we need to set up
   // a new source rectangle, and indicate the code for the proportions...
   //
   switch ( NewStyle )
      {

      case IDM_VDW_CLIENT_ARBITRARY_1   :
      case IDM_VDW_CLIENT_ARBITRARY_2   :
      case IDM_VDW_CLIENT_ARBITRARY_3   :
         NewSrcCode = NewStyle ;
         break;

      //case IDM_VDW_CHILD              :
      //case IDM_VDW_CLIENT_FULL_EXTENT :
      //case IDM_VDW_CLIENT_04TH_EXTENT :
      //case IDM_VDW_CLIENT_16TH_EXTENT :
      //case IDM_VDW_CLIENT_64TH_EXTENT :
      default:
         NewSrcCode = IDM_VDW_CLIENT_FULL_EXTENT ;
         break ;
      }

   // Set source rectangle based on the new style...
   pvi->rclCurSrc = pvi->prclSrcRects [ NewSrcCode - IDM_VDW_CHILD ] ;

   // Set source aspect ratio values from the original dimensions of the source...
   prclSrc         = &pvi->rclCurSrc ;
   pvi->SrcAspectX = ( prclSrc->xRight  - prclSrc->xLeft   ) + 1 ;
   pvi->SrcAspectY = ( prclSrc->yTop    - prclSrc->yBottom ) + 1 ;

   // If the source area is smaller than the destination area...
   SWidth  = ( prclSrc->xRight - prclSrc->xLeft   ) + 1 ;
   SHeight = ( prclSrc->yTop   - prclSrc->yBottom ) + 1 ;
   DWidth  = ( prclDst->xRight - prclDst->xLeft   ) + 1 ;
   DHeight = ( prclDst->yTop   - prclDst->yBottom ) + 1 ;
   if ( ( SWidth < DWidth ) || ( SHeight < DHeight ) )
      {

      // Make the source area the same size as the destination area...
      prclSrc->xLeft   = 8                ;
      prclSrc->yBottom = 4                ;
      prclSrc->xRight  = ( DWidth  + 8  ) - 1 ;
      prclSrc->yTop    = ( DHeight + 4  ) - 1 ;
      }

   // Make the MINIMUM SOURCE RECTANGLE the same size as the destination...
   prclMinSrc = &pvi->rclMinSrc  ;
   prclMinSrc->xLeft   = 0 ;
   prclMinSrc->yBottom = 0 ;
   if ( NewStyle == IDM_VDW_CHILD )
      {
      prclMinSrc->xRight  = pvi->prclDstRects
                               [IDM_VDW_CHILD - IDM_VDW_CHILD].xRight ;
      prclMinSrc->yTop    = pvi->prclDstRects
                               [IDM_VDW_CHILD - IDM_VDW_CHILD].yTop   ;
      }
   else
      {
      prclMinSrc->xRight  = DWidth  - 1 ;
      prclMinSrc->yTop    = DHeight - 1 ;
      }
   return TRUE ;
   }


//----------------------------------------------------------------------------
// Name:          InitializeVideoWindowInfo()
//
// Description:   Initialize overlay info...
//
// Concepts:      Sets initial values for our handling of overlay.
//
// Parameters:    pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
//                VideoInput - 1, 2, or 3
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID InitializeVideoWindowInfo ( PVIDEOWINDOWINFO pvi, LONG VideoInput )
   {
   PRECTL prcl ;
   DWORD  dwWidth, dwHeight ;

   // Set the video input to 3, 2, or 1...
   OvlSetInput ( paiMain->wOverlayDeviceID, VideoInput,
                         MCI_VIDEO_IN_CONNECTOR ) ;

   // Get the dimensions of the max video source buffer...
   OvlGetSrcExtRect  ( paiMain->wOverlayDeviceID, ( HWND ) 0,
                       &dwWidth, &dwHeight ) ;
   prcl          = &pvi->rclMaxSrc ;
   prcl->xLeft   = 0        ;
   prcl->yBottom = 0        ;
   prcl->xRight  = prcl->xLeft   + ( dwWidth  - 1 ) ;
   prcl->yTop    = prcl->yBottom + ( dwHeight - 1 ) ;

   // Set items based on PAL or NTSC...
   //
   //    -- Set input type to PAL or NTSC...
   //    -- Pointer to items associated w/ "Window" menu item:
   //       -- Source rectangles
   //       -- Destination rectangles  (proportional to source rects)
   //       -- Menu item strings
   //
   if ( pvi->rclMaxSrc.yTop < VVK_SRC_CY_MAX_NTSC )
      {
      pvi->InputType         = VVK_IT_NTSC ;
      pvi->prclSrcRects      = arclSrcRectsForNTSCInput ;
      pvi->prclDstRects      = arclDstRectsForNTSCInput ;
      pvi->ppszWindowStrings = apszWindowStringsForNTSC ;
      }
   else
      {
      pvi->InputType = VVK_IT_PAL ;
      pvi->prclSrcRects      = arclSrcRectsForPALInput  ;
      pvi->prclDstRects      = arclDstRectsForPALInput  ;
      pvi->ppszWindowStrings = apszWindowStringsForPAL  ;
      }

   // Initialize the max destination rectangle...same as max src...
   pvi->rclMaxDst = pvi->rclMaxSrc ;

   // Initialize the min video source buffer....use smallest possible coordinates...
   prcl          = &pvi->rclMinSrc ;
   prcl->xLeft   = 0        ;
   prcl->yBottom = 0        ;
   prcl->xRight  = VVK_SRC_CX_MIN - 1 ;
   prcl->yTop    = VVK_SRC_CY_MIN - 1 ;

   // Initialize the min destination rectangle...
   prcl          = &pvi->rclMinDst ;
   prcl->xLeft   = 0        ;
   prcl->yBottom = 0        ;
   prcl->xRight  = VVK_DST_CX_MIN - 1 ;
   prcl->yTop    = VVK_DST_CY_MIN - 1 ;

   // Set source aspect ratio values from the original dimensions of the source...
   prcl            = &pvi->rclCurSrc ;
   pvi->SrcAspectX = ( prcl->xRight  - prcl->xLeft   ) + 1 ;
   pvi->SrcAspectY = ( prcl->yTop    - prcl->yBottom ) + 1 ;

   // Set the deltas for source panning and scrolling...
   pvi->DeltaLineHorzPan  =  1  ;
   pvi->DeltaPageHorzPan  =  8  ;
   pvi->DeltaLineVertPan  =  1  ;
   pvi->DeltaPageVertPan  =  4  ;
   pvi->DeltaLineHorzZoom =  2  ;
   pvi->DeltaPageHorzZoom = 16  ;
   pvi->DeltaLineVertZoom =  2  ;
   pvi->DeltaPageVertZoom =  4  ;

   // Set the aspect ratio switch...a TRUE means that we will keep the
   // x-length and from it find the y-length...
   pvi->fAspectKeepX = TRUE ;

   // Set the information associated with the video window...
   InitializeVideoDisplayInfo ( pvi, VideoInput ) ;
   }


//-----------------------------------------------------------------------------
// Function:      InitializeVideoDisplayInfo()
//
// Description:   Initialize overlay info for display.
//
// Concepts:      Sets initial values for our handling of overlay.
//
// Parameters:    pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
//                VideoInput - 1, 2, or 3
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID InitializeVideoDisplayInfo ( PVIDEOWINDOWINFO pvi, LONG VideoInput )
   {
   PRECTL prcl ;
   LONG   MinWidth, MinHeight, AspectX, AspectY ;

   // Set the default style
   pvi->Style = IDM_VDW_CLIENT_04TH_EXTENT ;

   // Set current source and destination rects...
   pvi->rclCurSrc = pvi->prclSrcRects
                       [ IDM_VDW_CLIENT_04TH_EXTENT - IDM_VDW_CHILD ] ;
   pvi->rclCurDst = pvi->prclDstRects
                       [ IDM_VDW_CLIENT_04TH_EXTENT - IDM_VDW_CHILD ] ;

   // Set source aspect ratio values from the original dimensions of the source...
   prcl            = &pvi->rclCurSrc ;
   pvi->SrcAspectX = ( prcl->xRight  - prcl->xLeft   ) + 1 ;
   pvi->SrcAspectY = ( prcl->yTop    - prcl->yBottom ) + 1 ;


   // Determine the min client size for this particular application window...
   //
   // 1) Create a small rectangle and call WinCalcFrameRect() to get the
   //    minimum frame size...depends on number and size of menu items,
   //    as well as border size and header size...
   // 2) Now call WinCalcFrameRect() w/ TRUE to get minimum client size...
   // 3) Decrement xRight and yTop, since PM makes these 1 pel too big...
   // 4) Now determine the min client size proportional to the video source...
   //
   //
   pvi->rclMinClient.xLeft    = 0 ;
   pvi->rclMinClient.yBottom  = 0 ;
   pvi->rclMinClient.xRight   = 10 ;
   pvi->rclMinClient.yTop     = 10 ;
   WinCalcFrameRect ( hwndFrame, &pvi->rclMinClient, FALSE ) ;
   WinCalcFrameRect ( hwndFrame, &pvi->rclMinClient, TRUE  ) ;
   MinWidth  = ( pvi->rclMinClient.xRight - pvi->rclClient.xLeft ) + 1 ;
   AspectX   = pvi->SrcAspectX ;
   AspectY   = pvi->SrcAspectY ;
   MinHeight = ( ( MinWidth * AspectY ) + ( AspectX / 2 ) ) / AspectX ;
   pvi->rclMinClient.xLeft    = 0 ;
   pvi->rclMinClient.yBottom  = 0 ;
   pvi->rclMinClient.xRight   = MinWidth  - 1 ;
   pvi->rclMinClient.yTop     = MinHeight - 1 ;

   // Set the default client area, client on desktop, frame,
   // and frame on desktop...
   //
   // Initialize the client to the same size as the destination...
   pvi->rclClient          = pvi->rclCurDst ;

   // Intialize the size of this client on the desktop...
   pvi->rclClientOnDesktop = pvi->rclClient ;
   WinMapWindowPoints ( hwndClient, HWND_DESKTOP,
                       ( PPOINTL ) &pvi->rclClientOnDesktop, 2 ) ;

   // Initialize the frame rectangle...
   //    -- Copy the client size...
   //    -- Bump the xRight and yTop...For reasons that seemed good at
   //       the time, but no longer seem so good, we keep the client
   //       rectangle as (0,0,width-1,height-1) and the frame, in
   //       contrast, as (0,0,width,height)...So we have to make that
   //       adjustment here...
   //
   pvi->rclFrame = pvi->rclClient ;
   if ( VideoInput != 1)
      {
      pvi->rclFrame.xRight++ ;
      pvi->rclFrame.yTop++   ;
      }
   WinCalcFrameRect ( hwndFrame, &pvi->rclFrame, FALSE ) ;

   // Initialize the frame rectangle in desktop coords...
   //    -- First we have to shift right and up by the size of the
   //       border...Above, we have copied the frame size from the client size
   //       (which we do not do when the input is actually active),
   //       and applied WinCalcFrameRect().  Since the frame has a border,
   //       the rclClient (0,0,xxx,yyy) was transformed to the rclFrame
   //       (-leftborder, -bottomborder, www, zzz).  So now we make an
   //       adjustment to define the frame in non-negative coordinates.
   if ( VideoInput != 1)
      {
      pvi->rclFrame.xLeft   += paiMain->SizeBorderCx ;
      pvi->rclFrame.yBottom += paiMain->SizeBorderCy ;
      pvi->rclFrame.xRight  += paiMain->SizeBorderCx ;
      pvi->rclFrame.yTop    += paiMain->SizeBorderCy ;
      }
   pvi->rclFrameOnDesktop  = pvi->rclFrame ;
   WinMapWindowPoints ( hwndFrame, HWND_DESKTOP,
                       ( PPOINTL ) &pvi->rclFrameOnDesktop, 2 ) ;

   // Set other values...
   pvi->ActionMode         = VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO ;
   pvi->Readout            = VVK_RO_NONE ;
   pvi->clrClientBG        = CLR_BLACK   ;
   pvi->clrClientFG        = CLR_WHITE   ;
   }


//----------------------------------------------------------------------------
// Name:          UpdateSourceZoomCharacteristics
//
// Description:   Changes parms for zoom circular slider.
//
// Concepts:      When the dst rect changes (which in a normal app
//                we probably wd not allow to happen very often)
//                we have a new set of src-to-dst relationships
//                (dst size and shape, src size and shape); consequently
//                we have to re-initialize our controls.
//
// Parameters:    hwnd - the source control window
//                pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID UpdateSourceZoomCharacteristics ( HWND hwnd, PVIDEOWINDOWINFO pvi )
   {
   PRECTL   prclCur, prclMax, prclMin ;
   LONG     CurX, CurY, MaxX, MaxY, MinX, MinY, TryX, TryY  ;
   LONG     LoBound, HiBound, LineIncr, PageIncr, InitValue ;
   BOOL     fXLimitsMaxSize ;
   LONG     TestMinX, TestMinY, TestMaxX, TestMaxY ;

   // Get access to the rectangle values...
   prclCur     = &pvi->rclCurSrc ;
   prclMax     = &pvi->rclMaxSrc ;
   prclMin     = &pvi->rclMinSrc ;

   // Calculate source lengths...
   CurX = ( prclCur->xRight - prclCur->xLeft   ) + 1 ;
   MaxX = ( prclMax->xRight - prclMax->xLeft   ) + 1 ;
   MinX = ( prclMin->xRight - prclMin->xLeft   ) + 1 ;
   CurY = ( prclCur->yTop   - prclCur->yBottom ) + 1 ;
   MaxY = ( prclMax->yTop   - prclMax->yBottom ) + 1 ;
   MinY = ( prclMin->yTop   - prclMin->yBottom ) + 1 ;

   // Other init...
   fXLimitsMaxSize = TRUE ;

   // If we are keeping x in our aspect ratio calculations...
   // ...i.e. we take the width of the maximum source rectangle as constant,
   // and determine how large the height can become, rather than vice versa..
   //
   // E.g. Suppose the current source rectangle is 172 x 120, we are
   // keeping x, and the maximum source buffer size is 706 x 484.
   // How big can the y become under these circumstances? If the aspect
   // ratio of the destination is 43 x 30, and the max x value is 120,
   // then the max y is ( ( 706 * 3 ) / 4 ), or 529.  This value is larger
   // than the maximum destination size of y of 484, so the maximum source
   // will have to be limited along the Y axis, i.e. fXLimitsMaxSize = FALSE.
   //
   if ( pvi->fAspectKeepX )
      {
      // Determine how many clicks we need for the slider...
      //    -- What is the biggest possible x?..It is MaxX...
      //    -- Using MaxX and the aspect ratio, what is the biggest possible y?
      //    -- If the biggest y is less than max y...
      //       ...then let the max x determine the number of clicks...
      //       ...else let the max y determine the number of clicks...
      TryY = ( ( MaxX * pvi->SrcAspectY ) + ( pvi->SrcAspectX / 2 ) )
               / pvi->SrcAspectX ;
      if ( TryY > MaxY )
         {
         fXLimitsMaxSize = FALSE ;
         }
      }

   // Else do the same calculations for the other axis....
   else
      {
      TryX = ( ( MaxY * pvi->SrcAspectX ) + ( pvi->SrcAspectY / 2 ) )
               / pvi->SrcAspectY ;
      if ( TryX <= MaxX )
         {
         fXLimitsMaxSize = FALSE ;
         }
      }

   // Determine valid minimum and maximum values for x and y...
   //
   // The value of the slider represents pels...how many pels in the current
   // HEIGHT of the source rectangle (if the max size is limited along
   // the y axis) or how many pels in the WIDTH of the source rectangle
   // (if the max size is limited along the x axis).
   //
   // Example:  If the max source rectangle is 706 x 484, and the
   // max is limited along the y axis, then the minimum value of y
   // is 0 and the maximum is 484.
   //
   // Recall:  Absolute min and max is imposed by the hardware:
   //
   //
   //                   Min.    Max.
   //
   //          Horz:    4       706
   //
   //          Vert:    1       484
   //
   //
   // But the zoom software may impose slightly different limits
   // in order to observe aspect ratio requirements.  For example, if the
   // aspect ratio is 4 x 3, then when y is at its minimum of 1 ,
   // the minimum x will be
   //
   //       TestMinX = ( MinY * SrcAspectX ) / SrcAspectY
   //               = ( 1 * 4 ) / 3
   //               = 1 ;
   //
   // But 0 is an invalid value for x.  So, instead of using the lowest
   // possible minimum value for MinY (i.e. 1), we use the lowest
   // possible minimum value for MinY that gives a valid MinX:
   //
   //       MinX = ( MinY * pvi->SrcAspectX ) / pvi->SrcAspectY ;
   //
   //
   if ( fXLimitsMaxSize )
      {
      // Determine the minimum y value that could occur...
      // ...If it results in an invalid minimum y, then
      //    refigure minimum x as a function of minimum y...
      TestMinY = ( ( MinX * pvi->SrcAspectY ) + ( pvi->SrcAspectX / 2 ) )
                   / pvi->SrcAspectX ;
      if ( TestMinY < MinY )
         {
         MinX = ( ( MinY * pvi->SrcAspectX ) + ( pvi->SrcAspectY / 2 ) )
                  / pvi->SrcAspectY ;
         }

      // Same action for xmax and ymax
      TestMaxY = ( ( MaxX * pvi->SrcAspectY ) + ( pvi->SrcAspectX / 2 ) )
                   / pvi->SrcAspectX ;
      if ( TestMaxY > MaxY )
         {
         MaxX = ( ( MaxY * pvi->SrcAspectX ) + ( pvi->SrcAspectY / 2 ) )
                  / pvi->SrcAspectY ;
         }

      // Remember...
      LoBound = MinX ;
      HiBound = MaxX ;
      }
   else
      {
      // Determine the minimum x value that could occur...
      TestMinX = ( ( MinY * pvi->SrcAspectX ) + ( pvi->SrcAspectY / 2 ) )
                   / pvi->SrcAspectY ;
      if ( TestMinX < MinX )
         {
         MinY = ( ( MinX * pvi->SrcAspectY ) + ( pvi->SrcAspectX / 2 ) )
                  / pvi->SrcAspectX ;
         }

      // Same action for ymax and xmax...
      TestMaxX = ( ( MaxY * pvi->SrcAspectX ) + ( pvi->SrcAspectY / 2 ) )
                   / pvi->SrcAspectY ;
      if ( TestMaxX > MaxX )
         {
         MaxY = ( ( MaxX * pvi->SrcAspectY ) + ( pvi->SrcAspectX / 2 ) )
                  / pvi->SrcAspectX ;
         }

      // Remember....
      LoBound = MinY ;
      HiBound = MaxY ;
      }

   // Set the lower and upper limits for the slider...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_ZOOM_CONTROL ),
                CSM_SETRANGE,
                ( MPARAM ) LoBound,
                ( MPARAM ) HiBound ) ;

   // Determine and set the line and page increments...
   if ( fXLimitsMaxSize )
      {
      LineIncr = pvi->DeltaLineHorzZoom ;
      PageIncr = pvi->DeltaPageHorzZoom ;
      }
   else
      {
      LineIncr = pvi->DeltaLineVertZoom ;
      PageIncr = pvi->DeltaPageVertZoom ;
      }
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_ZOOM_CONTROL ),
                CSM_SETINCREMENT,
                ( MPARAM) PageIncr,
                ( MPARAM) LineIncr ) ;

   // Determine and set the initial value...
   if ( fXLimitsMaxSize )
      {
      InitValue = CurX ;
      }
   else
      {
      InitValue = CurY ;
      }
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_ZOOM_CONTROL ),
                CSM_SETVALUE,
                ( MPARAM) InitValue,
                ( MPARAM) NULL ) ;

   // Save values needed for later calculations...
   pvi->fSrcZoomXLimitsMaxSize = fXLimitsMaxSize ;
   pvi->SrcZoomLoBound         = LoBound        ;
   pvi->SrcZoomHiBound         = HiBound        ;
   pvi->SrcZoomPageIncr        = PageIncr       ;
   pvi->SrcZoomLineIncr        = LineIncr       ;
   }


//----------------------------------------------------------------------------
// Name:          UpdateHorzStretchCharacteristics()
//
// Description:   Updates circular slider for horizontal stretch.
//
// Concepts:      This function should always be called just AFTER,
//                rather than BEFORE, UpdatePanScrollBarCharacteristics(),
//                since it uses values calculated there -- Length and ThumbSize.
//
//                The stretch scroll bars use the SAME values as the corresponding
//                pan scroll bars.
//
// Parameters:    hwnd - the source control window
//
//                pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID UpdateHorzStretchCharacteristics ( HWND hwnd, PVIDEOWINDOWINFO pvi )
   {
   PRECTL   prclCur, prclMax, prclMin ;
   LONG     CurX, MaxX, MinX ;

   // Get access to the rectangle values...
   prclCur     = &pvi->rclCurSrc ;
   prclMax     = &pvi->rclMaxSrc ;
   prclMin     = &pvi->rclMinSrc ;

   // Calculate source lengths...
   CurX = ( prclCur->xRight - prclCur->xLeft   ) + 1 ;
   MaxX = ( prclMax->xRight - prclMax->xLeft   ) + 1 ;
   MinX = ( prclMin->xRight - prclMin->xLeft   ) + 1 ;

   // Set the lower and upper limits for the slider...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_STRETCH_HORZ ),
                CSM_SETRANGE,
                ( MPARAM ) MinX,
                ( MPARAM ) MaxX ) ;

   // Determine and set the line and page increments...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_STRETCH_HORZ ),
                CSM_SETINCREMENT,
                ( MPARAM) pvi->DeltaPageHorzZoom,
                ( MPARAM) pvi->DeltaLineHorzZoom ) ;

   // Determine and set the initial value...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_STRETCH_HORZ ),
                CSM_SETVALUE,
                ( MPARAM) CurX,
                ( MPARAM) NULL ) ;
   }


//----------------------------------------------------------------------------
// Name:          UpdateVertStretchCharacteristics()
//
// Description:   Updates circular slider for vertical stretch...
//
// Concepts:      This function should always be called just AFTER, rather
//                than BEFORE, UpdatePanScrollBarCharacteristics(), since
//                it uses values calculated there -- Length and ThumbSize.
//
//                The stretch scroll bars use the SAME values as the
//                corresponding pan scroll bars.
//
// Parameters:    hwnd - the source control window
//
//                pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID UpdateVertStretchCharacteristics ( HWND hwnd, PVIDEOWINDOWINFO pvi )
   {
   PRECTL   prclCur, prclMax, prclMin ;
   LONG     CurY, MaxY, MinY ;

   // Get access to the rectangle values...
   prclCur     = &pvi->rclCurSrc ;
   prclMax     = &pvi->rclMaxSrc ;
   prclMin     = &pvi->rclMinSrc ;

   // Calculate source lengths...
   CurY = ( prclCur->yTop - prclCur->yBottom ) + 1 ;
   MaxY = ( prclMax->yTop - prclMax->yBottom ) + 1 ;
   MinY = ( prclMin->yTop - prclMin->yBottom ) + 1 ;

   // Set the lower and upper limits for the slider...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_STRETCH_VERT ),
                CSM_SETRANGE,
                ( MPARAM ) MinY,
                ( MPARAM ) MaxY ) ;

   // Determine and set the line and page increments...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_STRETCH_VERT ),
                CSM_SETINCREMENT,
                ( MPARAM) pvi->DeltaPageVertZoom,
                ( MPARAM) pvi->DeltaLineVertZoom ) ;

   // Determine and set the initial value...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_STRETCH_VERT ),
                CSM_SETVALUE,
                ( MPARAM) CurY,
                ( MPARAM) NULL ) ;
   }


//----------------------------------------------------------------------------
// Name:          UpdatePanScrollBarCharacteristics()
//
// Description:   Update characteristics for pan scroll bars.
//
// Concepts:      Update both horz and vert.
//
// Parameters:    hwnd - the source control window
//
//                pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID UpdatePanScrollBarCharacteristics ( HWND hwnd, PVIDEOWINDOWINFO pvi )
   {
   PRECTL   prclCurrent, prclMax ;
   LONG     Length, ThumbPos, ThumbSize ;

   // Get access to the rectangle values...
   prclCurrent = &pvi->rclCurSrc ;
   prclMax     = &pvi->rclMaxSrc     ;

   // Figure the HORIZONTAL scroll bar thumbsize and length and store them...
   Length                      = ( prclMax->xRight - prclMax->xLeft ) + 1 ;
   ThumbSize                   = ( prclCurrent->xRight - prclCurrent->xLeft ) + 1 ;
   pvi->SrcHSBLength    = Length    ;
   pvi->SrcHSBThumbSize = ThumbSize ;

   // Figure the thumb position...
   ThumbPos = prclCurrent->xLeft ;

   // Set the scroll bar range and thumb...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_PAN_HORZ ),
                SBM_SETSCROLLBAR,
                MPFROM2SHORT ( ThumbPos, 0 ),
                MPFROM2SHORT ( 0, ( Length - ThumbSize ) - 1 ) ) ;

   // Set the thumb size...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_PAN_HORZ ),
                SBM_SETTHUMBSIZE,
                MPFROM2SHORT ( ThumbSize, Length ),
                0 ) ;

   // Figure the VERTICAL scroll bar length and thumb size and save these values...
   Length                      = ( prclMax->yTop - prclMax->yBottom ) + 1 ;
   ThumbSize                   = ( prclCurrent->yTop - prclCurrent->yBottom ) + 1 ;
   pvi->SrcVSBLength    = Length    ;
   pvi->SrcVSBThumbSize = ThumbSize ;

   // Figure the thumb position...
   ThumbPos = VVM_SOURCE_TO_VSCROLL ( Length, ThumbSize, prclCurrent->yBottom );

   // Set the scroll bar range and thumb...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_PAN_VERT ),
                SBM_SETSCROLLBAR,
                MPFROM2SHORT ( ThumbPos, 0 ),
                MPFROM2SHORT ( 0, ( Length - ThumbSize ) - 1 ) ) ;

   // Set the thumb size...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_PAN_VERT ),
                SBM_SETTHUMBSIZE,
                MPFROM2SHORT ( ThumbSize, Length ),
                0 ) ;
   }


//----------------------------------------------------------------------------
// Name:          SetVerticalScrollBarPosition()
//
// Description:   Sets vertical pan scroll bar position.
//
// Concepts:      ...
//
// Parameters:    hwnd - the source control window
//
//                pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID SetVerticalScrollBarPosition ( HWND hwnd, PVIDEOWINDOWINFO pvi )
   {
   LONG ThumbPos ;

   // ***Caution***: Leave this statement on ONE LINE:
   ThumbPos = VVM_SOURCE_TO_VSCROLL ( pvi->SrcVSBLength, pvi->SrcVSBThumbSize, pvi->rclCurSrc.yBottom ) ;

   // Set the position...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_PAN_VERT ),
                SBM_SETPOS,
                MPFROMSHORT ( ThumbPos ) ,
                NULL ) ;
   }


//----------------------------------------------------------------------------
// Name:          SetHorizontalScrollBarPosition()
//
// Description:   Sets horizontal pan scroll bar position.
//
// Concepts:      ...
//
// Parameters:    hwnd - the source control window
//
//                pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID SetHorizontalScrollBarPosition ( HWND hwnd, PVIDEOWINDOWINFO pvi )
   {
   LONG ThumbPos ;

   ThumbPos = pvi->rclCurSrc.xLeft ;

   // Set the position...
   WinSendMsg ( WinWindowFromID ( hwnd, ID_SRC_RCL_PAN_HORZ ),
                SBM_SETPOS,
                MPFROMSHORT ( ThumbPos ) ,
                NULL ) ;
   }


//----------------------------------------------------------------------------
// Name:          FitSegmentInsideSegment()
//
// Description:   Slides a rectangle either horz or vert so that its boundaries
//                are completely within the boundaries of another rectangle.
//
// Concept:       Shifts, if necessary, a shorter segment inside a longer
//                segment so that the shorter segment is entirely within
//                the bounds of the longer segment, without overlapping on
//                either side.
//
//                We use this function, for example, during a pan of the source
//                buffer, to keep the source rectangle from crossing the
//                valid boundaries of the source buffer.
//
// Parameters:    prclInput - input rectangle
//
//                prclMin - min size
//
//                prclMax - max size
//
//                prclResult - new rectangle shifted horz or vert
//
//                fVertical - if TRUE shift vert, else shift horz
//
// Returns:       TRUE if fit achieved, else FALSE
//
//----------------------------------------------------------------------------
BOOL FitSegmentInsideSegment ( PRECTL prclInput, PRECTL prclMin, PRECTL prclMax,
                               PRECTL prclResult, BOOL fVertical  )
   {
   LONG   x1Input, x2Input, x1Max, x2Max, x1Min, x2Min ;
   LONG   InputLength, MaxLength, MinLength, LoOverlap, HiOverlap ;
   LONG   x1Result, x2Result ;
   PLONG  px1Result, px2Result ;

   // Initialize...
   if ( fVertical )
      {
      x1Input   = prclInput->yBottom   ;
      x2Input   = prclInput->yTop      ;
      x1Max     = prclMax->yBottom     ;
      x2Max     = prclMax->yTop        ;
      x1Min     = prclMin->yBottom     ;
      x2Min     = prclMin->yTop        ;
      px1Result = &prclResult->yBottom ;
      px2Result = &prclResult->yTop    ;
      }
   else
      {
      x1Input  = prclInput->xLeft      ;
      x2Input  = prclInput->xRight     ;
      x1Max    = prclMax->xLeft        ;
      x2Max    = prclMax->xRight       ;
      x1Min    = prclMin->xLeft        ;
      x2Min    = prclMin->xRight       ;
      px1Result = &prclResult->xLeft   ;
      px2Result = &prclResult->xRight  ;
      }

   // Get the lengths...
   InputLength  = ( x2Input  - x1Input  ) + 1 ;
   MaxLength = ( x2Max - x1Max ) + 1 ;
   MinLength = ( x2Min - x1Min ) + 1 ;

   // Do sanity checks...
   if ( ( InputLength > MaxLength ) || ( InputLength < MinLength ) )
      {
      return FALSE ;
      }

   // Get overlaps at lower and higher bounds...
   LoOverlap = x1Input  - x1Max ;
   HiOverlap = x2Max - x2Input  ;

   // If lower bound overlaps...then shift it...
   if ( LoOverlap < 0 )
      {
      x1Result = x1Max ;
      x2Result = ( x1Result + InputLength ) - 1 ;
      }

   // Else if upper bound overlaps...then shift it...
   else if ( HiOverlap < 0 )
      {
      x2Result = x2Max ;
      x1Result = ( x2Result - InputLength ) + 1 ;
      }

   // Else leave the Input segment where it lies...
   else
      {
      x1Result = x1Input ;
      x2Result = x2Input ;
      }

   // Set return values...and return...
   *prclResult = *prclInput ;
   *px1Result  = x1Result ;
   *px2Result  = x2Result ;
   return TRUE ;
   }


//----------------------------------------------------------------------------
// Name:          FitRectInsideRect()
//
// Description:   Moves a smaller rectangle until it no longer overlaps the edge
//                of the large rectangle that it lies in.
//
// Concepts:      Same as FitSegmentInsideSegment(), wh this function calls
//                twice, 1 for horz and 1 for vert.
//
// Parameters:    prclInput - input rectangle
//
//                prclResult - new rectangle shifted horz or vert
//
//                prclMin - min size
//
//                prclMax - max size
//
// Returns:       TRUE if fit achieved, else FALSE
//
//----------------------------------------------------------------------------
BOOL FitRectInsideRect ( PRECTL prclInput, PRECTL prclResult,
                         PRECTL prclMin, PRECTL prclMax )
   {
   RECTL rclResult1, rclResult2 ;
   BOOL  fSuccess ;

   // X-axis...
   fSuccess = FitSegmentInsideSegment ( prclInput, prclMin, prclMax,
                                        &rclResult1, FALSE ) ;
   if ( !fSuccess )
      {
      return FALSE ;
      }

   // Y-axis...
   fSuccess = FitSegmentInsideSegment ( &rclResult1, prclMin, prclMax,
                                        &rclResult2, TRUE );
   if ( !fSuccess )
      {
      return FALSE ;
      }

   // Done...
   *prclResult = rclResult2 ;
   return TRUE ;
   }


//----------------------------------------------------------------------------
// Name:          DisplaySourceTextValues()
//
// Description:   Update values for readout display.
//
// Concepts:      These values are displayed in the upper left quadrant of
//                the Adjust Source Values dialog box.
//
// Parameters:    hwnd - window being painted
//
//                prcl - rectangle whose values are being displayed...
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID DisplaySourceTextValues ( HWND hwnd, PRECTL prcl )
   {
   BYTE abTemp [ 20 ] ;

   // Update the cy field...
   sprintf ( abTemp, "cy: %d", ( prcl->yTop - prcl->yBottom ) + 1 ) ;
   WinSetDlgItemText ( hwnd, ID_SRC_RCL_TEXT_CY, abTemp ) ;

   // Update the x2, y2 field...
   sprintf ( abTemp, "(%d,%d)", prcl->xRight, prcl->yTop ) ;
   WinSetDlgItemText ( hwnd, ID_SRC_RCL_TEXT_X2_Y2, abTemp ) ;

   // Update the x1, y1 field...
   sprintf ( abTemp, "(%d,%d)", prcl->xLeft, prcl->yBottom ) ;
   WinSetDlgItemText ( hwnd, ID_SRC_RCL_TEXT_X1_Y1, abTemp ) ;

   // Update the cx field...
   sprintf ( abTemp, "cx: %d", ( prcl->xRight - prcl->xLeft ) + 1 ) ;
   WinSetDlgItemText ( hwnd, ID_SRC_RCL_TEXT_CX, abTemp ) ;
   }


//----------------------------------------------------------------------------
// Name:          ApplyAspectRatio()
//
// Description:   Applies specified aspect ratio to specified rectangle...
//
// Concepts:      Either the width or the height of the input rect is
//                to remain the same, while the other dimension changes.
//
// Parameters:    XAspect, YAspect - the components of the aspect ratio --
//                                   usually the literal dimensions of
//                                   the rectangle serving as a model
//
//                prclIn - input rectangle
//
//                prclIn - output rectangle
//
//                fAspectKeepX - which dimension stays the same
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID ApplyAspectRatio ( LONG XAspect, LONG YAspect, PRECTL prclIn,
                        PRECTL prclOut, BOOL fAspectKeepX )
   {
   LONG  XLen, YLen ;

   // Initialize output rectangle...
   *prclOut = *prclIn ;

   // If the x-length is to remain the same, while the y-length changes...
   if ( fAspectKeepX )
      {
      XLen = ( prclIn->xRight - prclIn->xLeft   ) + 1 ;
      YLen = ( ( XLen * YAspect ) + ( XAspect / 2 ) ) / XAspect ;
      }

   // Else the y-length is to remain the same, while the x-length changes...
   else
      {
      YLen = ( prclIn->yTop   - prclIn->yBottom ) + 1 ;
      XLen = ( ( YLen * XAspect ) + ( YAspect / 2 ) ) / YAspect ;
      }

   // Update output rectangle...
   prclOut->xRight = ( prclOut->xLeft   + XLen ) - 1 ;
   prclOut->yTop   = ( prclOut->yBottom + YLen ) - 1 ;
   }


//----------------------------------------------------------------------------
// Name:          UpdateSourceArea()
//
// Description:   Change source rectangle and readouts.
//
// Concepts:      Make sure the source rectangle size and the
//                upper-left-quadrant of the Adjust Source Area
//                dialog change at the same time.
//
// Parameters:    hwnd - hwnd of dialog
//
//                pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID UpdateSourceArea ( HWND hwnd, PVIDEOWINDOWINFO pvi )
   {

   // Change the source rectangle...
   OvlPutSrcRect ( paiMain->wOverlayDeviceID, &pvi->rclCurSrc, ( HWND ) 0 ) ;

   // Display the new values...
   DisplaySourceTextValues ( hwnd, &pvi->rclCurSrc ) ;

   // If the child of the client is the video window, and the readouts
   // are active, then force a paint so that the readouts will be
   // updated.
   if (    pvi->Style == IDM_VDW_CHILD
        && pvi->Readout != VVK_RO_NONE )
      {
      WinInvalidateRect ( hwndClient, NULL, FALSE ) ;
      WinUpdateWindow   ( hwndClient ) ;
      }
   }


//-----------------------------------------------------------------------------
// Name:          AddLengthToRect()
//
// Description:   Grows a rectangle vertically or horizontally.
//
// Concepts:      Used to "zoom" or "stretch" the source rectangle
//                larger or smaller.  We split the delta and add
//                the halves to either end.
//
// Parameters:    prclInput - input
//
//                prclOutput - result after growth
//
//                Delta - positive or negative integer
//
//                fAddToWidth - which dimension to add Delta to
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID AddLengthToRect ( PRECTL prclInput, PRECTL prclResult, LONG Delta,
                         BOOL fAddToWidth )
{
   PLONG    pLoBound, pHiBound ;
   LONG     LoPart, HiPart ;

   // Init...
   *prclResult = *prclInput ;

   // If adding to the width...
   if ( fAddToWidth )
      {
      // Get useful values...
      pLoBound     = &prclResult->xLeft  ;
      pHiBound     = &prclResult->xRight ;

      // Split the change between the low bound and the high bound..
      LoPart = Delta / 2 ;         // For example,  7/2 = 3.
      HiPart = LoPart ;            // Therefore LoPart = 3 and HiPart = 3
      if ( ( Delta % 2 ) != 0 )        //  7 % 2 is 1.
         {
         if ( Delta > 0 )
            {
            HiPart += 1 ;              // Here +3 would become +4...
            }
         else
            {
            HiPart -= 1 ;              // Here -3 would become -4...
            }
         }
      }

   // Else adding to the height...
   else
      {
      // Get useful values...
      pLoBound     = &prclResult->yBottom ;
      pHiBound     = &prclResult->yTop    ;

      // Y values must be multiples of 1...
      // ...

      // Split the change between the low bound and the high bound..
      LoPart = Delta / 2 ;             // For example,  7/2 = 3.
      HiPart = LoPart ;                // Therefore LoPart = 3 and HiPart = 3.
      if ( ( Delta % 2 ) != 0 )        // 7 % 2 is 1.
         {
         if ( Delta > 0 )
            {
            HiPart += 1 ;              // Here +3 would become +4...
            }
         else
            {
            HiPart -= 1 ;              // Here -3 would become -4...
            }
         }
      }

   // Adjust the length on both ends...
   //
   // -- if ( Delta > 0 ) then LoPart and HiPart are both > 0
   //    and segment "aaaaaaaa" is adjusted to become "baaaaaaaab"
   //
   // -- if ( Delta < 0 ) then LoPart and HiPart are both < 0
   //    and segment "aaaaaaaa" is adjusted to become "aaaaaa"
   //
   // ...The math works out correctly either way...
   *pLoBound -= LoPart ;
   *pHiBound += HiPart ;
}


//-----------------------------------------------------------------------------
// Name:          MapInvalidRectToValidRect()
//
// Description:   Clips height and width to specified max and min.
//
// Concepts:      Refuses to let input rectangle shrink inside or grow
//                outside stated bounds.
//
//                Used to keep our source rect from getting SMALLER than
//                our dest rect or LARGER than our source buffer.
//
// Parameters:    prclCur - input
//
//                prclResult - output after mapping
//
//                prclMin - the min
//
//                prclMax - the max
//
//                fMapX - whether mapping horz or vert
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID MapInvalidRectToValidRect ( PRECTL prclCur, PRECTL prclResult,
                           PRECTL prclMin, PRECTL prclMax, BOOL fMapX )
{
   PLONG  pHiBound, pLoBound ;
   LONG   CurLength, MinLength, MaxLength ;

   // Initialize result...
   *prclResult = *prclCur ;

   // Set up values...
   if ( fMapX )
      {
      MaxLength = ( prclMax->xRight - prclMax->xLeft ) + 1 ;
      MinLength = ( prclMin->xRight - prclMin->xLeft ) + 1 ;
      CurLength = ( prclCur->xRight - prclCur->xLeft ) + 1 ;
      pLoBound  = &prclResult->xLeft   ;
      pHiBound  = &prclResult->xRight  ;
      }
   else
      {
      MaxLength = ( prclMax->yTop - prclMax->yBottom ) + 1 ;
      MinLength = ( prclMin->yTop - prclMin->yBottom ) + 1 ;
      CurLength = ( prclCur->yTop - prclCur->yBottom ) + 1 ;
      pLoBound  = &prclResult->yBottom ;
      pHiBound  = &prclResult->yTop    ;
      }

   // Fix the length if necessary....
   if ( CurLength < MinLength )
      {
      *pHiBound = *pLoBound + MinLength ;
      }
   else if ( CurLength > MaxLength )
      {
      *pHiBound = *pLoBound + MaxLength ;
      }
   }


//-----------------------------------------------------------------------------
// Name:          ApplyZoomToRect()
//
// Description:   Enlarge or shrink the specified rectangle, maintaining
//                the specified aspect ratio.
//
// Concepts:      Does zoom processing.
//
// Parameters:    prclInput - input
//
//                prclResult - output after mapping
//
//                NewLength1 - value we are zooming to
//
//                fXLimitsMaxSize - In which dimension, vert or horz, we
//                                  will sooner run out of room to expand;
//                                  depends on the limiting size and shape
//                                  of the source rectangle, and the
//                                  of our destination rect.
//
//                AspectX, AspectY - components of aspect ratio
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID ApplyZoomToRect ( PVIDEOWINDOWINFO pvi, PRECTL prclInput,PRECTL prclResult,
                            LONG NewLength1, BOOL fXLimitsMaxSize,
                            LONG AspectX, LONG AspectY )
{
   LONG     Width, Height, OldLength1, OldLength2, NewLength2 ;
   RECTL    rclResult1, rclResult2, rclResult3 ;

   // Init...
   Width  = ( prclInput->xRight - prclInput->xLeft   ) + 1 ;
   Height = ( prclInput->yTop   - prclInput->yBottom ) + 1 ;

   // STEP 1: Adjust along 1 axis...
   // Note: If x limits the max size...i.e.  if proportionally increasing
   //       the sides of the source rectangle causes us to run out of
   //       space on the x-axis before we run out of space on the y-axis...
   //       then adjust the x-side first...else adjust the y-side first...
   //
   if ( fXLimitsMaxSize )
      {
      OldLength1 = Width ;
      }
   else
      {
      OldLength1 = Height ;
      }
   AddLengthToRect ( prclInput, &rclResult1, NewLength1 - OldLength1,
                     fXLimitsMaxSize ) ;

   // STEP 2: Map an out-of-range length to the nearest valid length...
   MapInvalidRectToValidRect ( &rclResult1,
                               &rclResult2,
                               &pvi->rclMinSrc,
                               &pvi->rclMaxSrc,
                               fXLimitsMaxSize ) ;


   // STEP 3: Adjust along the other axis...using the previous dimensions
   //         of the rectangle as the aspect ratio...
   //
   // If we were working on the x-axis we now begin on the y-axis...
   if ( fXLimitsMaxSize )
      {
      OldLength2 = Height ;
      NewLength2 = ( ( NewLength1 * AspectY ) + ( AspectX / 2 ) ) / AspectX ;
      }

   // Else we were working on the y-axis so we now begin on the x-axis...
   else
      {
      OldLength2 = Width ;
      NewLength2 = ( ( NewLength1 * AspectX ) + ( AspectY / 2 ) ) / AspectY ;
      }
   AddLengthToRect ( &rclResult2, &rclResult3, NewLength2 - OldLength2,
                     fXLimitsMaxSize? 0 : 1 );

   // Done...return...
   *prclResult = rclResult3 ;
}


//-----------------------------------------------------------------------------
// Name:          ApplyStretchToRect()
//
// Description:   Like zoom but affects only 1 dimension...
//
// Concepts:      Does stretch processing
//
// Parameters:    prclInput - input
//
//                prclResult - output after mapping
//
//                NewLength1 - value we are stretching
//
//                fApplyToWidth - whether stretching width or height
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID ApplyStretchToRect ( PVIDEOWINDOWINFO pvi, PRECTL prclInput,PRECTL prclResult, LONG NewLength,
                          BOOL fApplyToWidth )
{
RECTL    rclResult1, rclResult2 ;
LONG     OldLength ;

   // Init...
   if ( fApplyToWidth )
      {
      OldLength  = ( prclInput->xRight - prclInput->xLeft   ) + 1 ;
      }
   else
      {
      OldLength = ( prclInput->yTop   - prclInput->yBottom ) + 1 ;
      }

   // Add the length to the rectangle...
   AddLengthToRect ( prclInput, &rclResult1, NewLength - OldLength,
                     fApplyToWidth ) ;


   // Map an out-of-range dimension to the nearest valid dimension...
   MapInvalidRectToValidRect ( &rclResult1,
                               &rclResult2,
                               &pvi->rclMinSrc,
                               &pvi->rclMaxSrc,
                               fApplyToWidth ) ;

   // Stretch affects only 1 dimension at a time...so no need to
   // figure the second dimension by using the first dimension and
   // the aspect ratio.

   // Done...return...
   *prclResult = rclResult2 ;
}


//----------------------------------------------------------------------------
// Name:          AttemptVerticalPan
//
// Description:   Slides rectangle up or down within enclosing rectangle.
//
// Concepts:      Used to slide our source rect within source buffer.
//
// Parameters:    hwnd - handle of Adjust Source Area rectangle
//
//                mp2 - mp2 value from WM_VSCROLL message
//
// Returns:       TRUE if successful, else FALSE
//
//----------------------------------------------------------------------------
BOOL AttemptVerticalPan ( HWND hwnd, MPARAM mp2, PVIDEOWINDOWINFO pvi )
   {
   RECTL    rclProposed, rclResult ;
   LONG     Delta ;
   BOOL     fSuccess, fDoPan ;

   // Initialize...
   fDoPan      = TRUE ;
   rclProposed = pvi->rclCurSrc ;

   // Get the scroll bar command...
   switch ( SHORT2FROMMP ( mp2 ) )
      {

      case SB_LINEUP   :
         Delta = + ( pvi->DeltaLineVertPan ) ;
         break;

      case SB_LINEDOWN :
         Delta = - ( pvi->DeltaLineVertPan ) ;
         break;

      case SB_PAGEUP   :
         Delta = + ( pvi->DeltaPageVertPan ) ;
         break;

      case SB_PAGEDOWN :
         Delta = - ( pvi->DeltaPageVertPan ) ;
         break;

      case SB_SLIDERPOSITION :
         fDoPan = FALSE ;
         break;

      case SB_SLIDERTRACK :

         // Flip the thumb value, since we are treating this
         // scroll bar as "upside down"...
         Delta = VVM_VSCROLL_TO_SOURCE( pvi->SrcVSBLength, pvi->SrcVSBThumbSize, SHORT1FROMMP ( mp2 ) ) ;

         // Delta is ( new position - old position )...
         Delta = Delta - rclProposed.yBottom ;

         // Adjust for 1-pel boundary...
         if ( Delta > 0 )
            {
            Delta += 1 ;
            }
         else
            {
            Delta -= 1 ;
            }
         break;

      case SB_ENDSCROLL :
         fDoPan = FALSE ;
         break;
      }   // End of command switch...

   // If we are to make a change...
   if ( fDoPan )
      {

      // Determine whether the proposed shift can be fitted
      // into the video source buffer...
      rclProposed.yBottom += Delta ;
      rclProposed.yTop    += Delta ;
      fSuccess = FitSegmentInsideSegment ( &rclProposed,
                                &pvi->rclMinSrc,
                                &pvi->rclMaxSrc,
                                &rclResult, TRUE ) ;

      // If the fit was successful...
      // ...Copy the change into the current source rectangle...
      // and return success...
      if ( fSuccess )
         {
         pvi->rclCurSrc = rclResult ;
         return TRUE ;
         }
      }
   return FALSE ;
   }


//----------------------------------------------------------------------------
// Name:          AttemptHorizontalPan
//        ..
// Description:   Slides rectangle left or right within enclosing rectangle...
//
// Concepts:      Used to slide our source rect within source buffer.
//
// Parameters:    hwnd - handle of Adjust Source Area rectangle
//
//                mp2 - mp2 value from WM_VSCROLL message
//
// Returns:       TRUE if successful, else FALSE
//
//----------------------------------------------------------------------------
BOOL AttemptHorizontalPan ( HWND hwnd, MPARAM mp2, PVIDEOWINDOWINFO pvi )
   {
   RECTL    rclProposed, rclResult ;
   LONG     Delta ;
   BOOL     fSuccess, fDoPan ;

   // Initialize...
   fDoPan      = TRUE ;
   rclProposed = pvi->rclCurSrc ;

   // Get the scroll bar command...
   switch ( SHORT2FROMMP ( mp2 ) )
      {
      case SB_LINERIGHT   :
         Delta = + ( pvi->DeltaLineHorzPan ) ;
         break;

      case SB_LINELEFT :
         Delta = - ( pvi->DeltaLineHorzPan ) ;
         break;

      case SB_PAGERIGHT   :
         Delta = + ( pvi->DeltaPageHorzPan ) ;
         break;

      case SB_PAGELEFT :
         Delta = - ( pvi->DeltaPageHorzPan ) ;
         break;

      case SB_SLIDERPOSITION :
         fDoPan = FALSE ;
         break;

      case SB_SLIDERTRACK :
         // Delta is ( new position - old position )...
         Delta = SHORT1FROMMP ( mp2 ) - rclProposed.xLeft ;
         break;

      case SB_ENDSCROLL :
         fDoPan = FALSE ;
         break;
      }   // End of command switch...

   // If we are to make a change...
   if ( fDoPan )
      {

      // Determine whether the proposed shift can be fitted
      // into the video source buffer...
      rclProposed.xLeft   += Delta ;
      rclProposed.xRight  += Delta ;
      fSuccess = FitSegmentInsideSegment ( &rclProposed,
                                &pvi->rclMinSrc,
                                &pvi->rclMaxSrc,
                                &rclResult, FALSE ) ;

      // If the fit was successful...
      // ...Copy the change into the current source rectangle...
      // and return success...
      if ( fSuccess )
         {
         pvi->rclCurSrc = rclResult ;
         return TRUE ;
         }
      }
   return FALSE ;
   }


//----------------------------------------------------------------------------
// Name:          AttemptZoom
//
// Description:   Try stretching horizontally and vertically with given
//                aspect ratio.
//
// Concepts:      High level zooming function.
//
// Parameters:    hwnd - handle of Adjust Source Area rectangle
//
//                mp2 - mp2 value from WM_CONTROL message
//
// Returns:       TRUE if successful, else FALSE
//
//----------------------------------------------------------------------------
BOOL AttemptZoom ( HWND hwnd, MPARAM mp2, PVIDEOWINDOWINFO pvi )
   {
   RECTL    rclCurrent, rclResult1, rclResult2 ;
   BOOL     fSuccess ;

   // Initialize...
   rclCurrent = pvi->rclCurSrc ;

   // Apply the change to a temporary rectangle...
   ApplyZoomToRect ( pvi,
                     &rclCurrent,
                     &rclResult1,
                     SHORT1FROMMP ( mp2 ),
                     pvi->fSrcZoomXLimitsMaxSize,
                     pvi->SrcAspectX,
                     pvi->SrcAspectY ) ;

   // Determine whether the proposed zoom can be
   // fitted into the video source buffer...
   fSuccess = FitRectInsideRect (
                    &rclResult1,
                    &rclResult2,
                    &pvi->rclMinSrc,
                    &pvi->rclMaxSrc ) ;

   // If the fit was successful...
   // ...Copy the change into the current source rectangle...
   // and return success...
   if ( fSuccess )
      {
      pvi->rclCurSrc = rclResult2 ;
      return TRUE ;
      }
   return FALSE ;
   }


//----------------------------------------------------------------------------
// Name:          AttemptStretch
//
// Description:   Try stretching horizontally or vertically independently
//                of any aspect ratio.
//
// Concepts:      High level zooming function.
//
// Parameters:    hwnd - handle of Adjust Source Area rectangle
//
//                mp2 - mp2 value from WM_CONTROL message
//
// Returns:       TRUE if successful, else FALSE
//
//----------------------------------------------------------------------------
BOOL AttemptStretch ( HWND hwnd, MPARAM mp2, PVIDEOWINDOWINFO pvi, BOOL fApplyToWidth )
   {
   RECTL    rclCurrent, rclResult1, rclResult2 ;
   BOOL     fSuccess ;


   // Initialize...
   rclCurrent = pvi->rclCurSrc ;

   // Apply the change to a temporary rectangle...
   ApplyStretchToRect ( pvi, &rclCurrent, &rclResult1,
                        SHORT1FROMMP ( mp2 ), fApplyToWidth ) ;

   // Determine whether the proposed stretch can be
   // fitted into the video source buffer...
   fSuccess = FitRectInsideRect (
                    &rclResult1,
                    &rclResult2,
                    &pvi->rclMinSrc,
                    &pvi->rclMaxSrc ) ;

   // If the fit was successful...
   // ...Copy the change into the current source rectangle...
   // and return success...
   if ( fSuccess )
      {
      pvi->rclCurSrc = rclResult2 ;
      return TRUE ;
      }
   return FALSE ;
   }


//----------------------------------------------------------------------------
// Name:          ChangeVideoWindowStyle()
//
// Description:   Change type or shape of video window...
//
// Concepts:      We allow different sizes and shapes of destination.
//                When the destination changes, we have to make
//                adjustments in our client area, controls, and colors.
//
// Parameters:    NewStyle - the new style selected from the menu.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
VOID ChangeVideoWindowStyle( PVIDEOWINDOWINFO pvi, LONG OldStyle, LONG NewStyle )
   {
   RECTL    rclNewClient, rclNewFrame, rclDlg, rclOldClient ;
   PRECTL   prcl, prclDst, prclDlg, prclFrame ;
   LONG     Width, Height, NewWidth, NewHeight ;
   HWND     hwnd, hwndDlg ;
   BOOL     fChangeSrcArea ;
   HWND     hwndVideo ;

   // If the client size is to be changed...
   if ( NewStyle != OldStyle )
      {
      // Get the client size in world coordinates...
      rclOldClient = pvi->rclClientOnDesktop ;
      rclNewClient = pvi->rclClientOnDesktop ;
      prcl         = &rclNewClient ;

      // Get a pointer to the default destination rectangle for this
      // style...
      if ( NewStyle == IDM_VDW_CHILD )   // Child uses 1/4 screen client...
         {
         prclDst = &pvi->prclDstRects
                       [ IDM_VDW_CLIENT_04TH_EXTENT - IDM_VDW_CHILD ] ;
         }
      else
         {
         prclDst = &pvi->prclDstRects [ NewStyle - IDM_VDW_CHILD ] ;
         }

      // Now set the values for the client area...
      //
      // First, if we are switching to full screen, then set xLeft
      // and yBottom to 0, otherwise use the current values...
      if ( NewStyle == IDM_VDW_CLIENT_FULL_EXTENT )
         {
         prcl->xLeft   = 0 ;
         prcl->yBottom = 0 ;
         }

      // Get the default width and height...
      Width  = ( prclDst->xRight  - prclDst->xLeft   ) + 1 ;
      Height = ( prclDst->yTop    - prclDst->yBottom ) + 1 ;

      // Now set the xRight and yBottom values...
      prcl->xRight  = prcl->xLeft   + ( Width  - 1 ) ;
      prcl->yTop    = prcl->yBottom + ( Height - 1 ) ;

      // Calculate the corresponding frame rectangle...still in
      // world coordinates...
      rclNewFrame = rclNewClient ;
      prclFrame   = &rclNewFrame ;
      WinCalcFrameRect ( hwndFrame, prclFrame, FALSE ) ;

      // Get the new width and height for the frame...
      NewWidth  = ( prclFrame->xRight  - prclFrame->xLeft   ) + 1 ;
      NewHeight = ( prclFrame->yTop    - prclFrame->yBottom ) + 1 ;

      // Make the client adjacent to the source-adjust-dialog...
      hwndDlg = paiMain->hwndSourceDlg ;
      if ( hwndDlg != 0 && NewStyle != IDM_VDW_CLIENT_FULL_EXTENT )
         {

         // Get values for dialog rectangle...
         prclDlg = &rclDlg ;
         WinQueryWindowRect ( hwndDlg, prclDlg ) ;
         WinMapWindowPoints ( hwndDlg, HWND_DESKTOP, ( PPOINTL ) prclDlg, 2 ) ;

         // Determine new position...
         if ( prclFrame->xLeft < prclDlg->xLeft )
            {
            prclFrame->xLeft   = prclDlg->xLeft - NewWidth ;
            prclFrame->yBottom = prclDlg->yTop - NewHeight ;
            }
         else
            {
            prclFrame->xLeft   = prclDlg->xRight           ;
            prclFrame->yBottom = prclDlg->yTop - NewHeight ;
            }
         }

      // else...
      else if ( NewStyle != IDM_VDW_CLIENT_FULL_EXTENT )
         {
         prclFrame->xLeft   = rclOldClient.xLeft   ;
         prclFrame->yBottom = rclOldClient.yBottom ;
         }

      // Set the frame to the newly calculated position...
      WinSetWindowPos ( hwndFrame, 0L ,
                        prclFrame->xLeft,
                        prclFrame->yBottom,
                        NewWidth, NewHeight,
                        SWP_MOVE | SWP_SIZE ) ;

      // Query and store the new frame position....
      WinQueryWindowRect ( hwndFrame, &pvi->rclFrame ) ;

      // If we are switching TO or FROM a child window...
      if (   ( NewStyle == IDM_VDW_CHILD )
          || ( OldStyle == IDM_VDW_CHILD ) )
         {

         // If switching TO...make required changes...
         //    -- Client background is white...
         //    -- Choose alternate foreground to avoid white-on-white...
         //    -- Disable the white-foreground alternative on the menu list...
         //    -- Show the child...
         //    -- New video window is child...
         //
         if ( NewStyle == IDM_VDW_CHILD )
            {
            pvi->clrClientBG = CLR_WHITE ;
            WinEnableMenuItem ( hwndMenu, IDM_RDTCLR_WHITE, FALSE ) ;
            pvi->SaveReadout = pvi->Readout ;
            if ( pvi->Readout == VVK_RO_WHITE )
               {
               pvi->Readout = VVK_RO_BLACK ;
               MapReadoutToClr ( VVK_RO_BLACK, &pvi->clrClientFG ) ;
               WinCheckMenuItem ( hwndMenu, IDM_RDTCLR_NONE + pvi->SaveReadout,
                                  FALSE ) ;
               WinCheckMenuItem ( hwndMenu, IDM_RDTCLR_BLACK, TRUE ) ;
               }
            hwndVideo = hwndChild ;
            }

         // Else switching FROM...make required changes...
         //    -- Client background is pink...
         //    -- No requirement for alternate forground...
         //    -- Re-enable white-foreground alternative on the menu list...
         //    -- Hide the child...
         //    -- New video window is client...
         else
            {
            pvi->clrClientBG = CLR_PINK ;
            WinEnableMenuItem ( hwndMenu, IDM_RDTCLR_WHITE, TRUE ) ;
            if (   pvi->SaveReadout != VVK_RO_NONE
                && pvi->Readout != pvi->SaveReadout )
               {
               WinCheckMenuItem ( hwndMenu, IDM_RDTCLR_NONE + pvi->Readout,
                                  FALSE ) ;
               WinCheckMenuItem ( hwndMenu, IDM_RDTCLR_NONE + pvi->SaveReadout,
                                  TRUE ) ;
               MapReadoutToClr ( pvi->SaveReadout, &pvi->clrClientFG ) ;
               pvi->Readout = pvi->SaveReadout ;
               }
            hwndVideo = hwndClient ;
            }

         // Force a fill of the client area...
         fDoFill = TRUE ;

         // If going TO child...then refocus video and show the window...
         if ( NewStyle == IDM_VDW_CHILD )
            {

            // Set a new video window...this takes a little time...and
            // show the child...
            OvlSetVideoWindow ( paiMain->wOverlayDeviceID, hwndVideo ) ;
            WinSetWindowPos ( hwndChild, 0L , 0, 0, 0, 0, SWP_SHOW ) ;
            }

         // Else going FROM child...then hide it now...
         else
            {
            WinSetWindowPos ( hwndChild, 0L , 0, 0, 0, 0, SWP_HIDE ) ;
            OvlSetVideoWindow ( paiMain->wOverlayDeviceID, hwndVideo ) ;
            }
         }


      // Determine whether we need to change the source area...
      fChangeSrcArea = MakeSrcPropToNewDst ( pvi, OldStyle, NewStyle ) ;

      // If the source area is to be changed...then make the change and
      // update the controls...
      if ( fChangeSrcArea )
         {
         hwnd = paiMain->hwndSourceDlg ;
         UpdateSourceArea ( hwnd, pvi ) ;
         UpdatePanScrollBarCharacteristics ( hwnd, pvi ) ;
         UpdateSourceZoomCharacteristics   ( hwnd, pvi ) ;
         UpdateHorzStretchCharacteristics  ( hwnd, pvi ) ;
         UpdateVertStretchCharacteristics  ( hwnd, pvi ) ;
         }

      // Save the new style...
      pvi->Style = NewStyle ;

      // If the readouts are active then force a paint of the
      // client area
      if ( pvi->Readout != VVK_RO_NONE )
         {
         WinInvalidateRect ( hwndClient, NULL, FALSE ) ;
         WinUpdateWindow   ( hwndClient ) ;
         }

      // If we are changing to full screen...
      if ( NewStyle == IDM_VDW_CLIENT_FULL_EXTENT )
         {
         WinMessageBox ( HWND_DESKTOP, HWND_DESKTOP, (PCH) szFullScreenMessage,
            ( PCH ) "Full Screen Client Area", ID_MESSAGEBOX,
            MB_OK | MB_SYSTEMMODAL | MB_ICONEXCLAMATION );
         }

      // Uncheck the menu item for the old style...and check the menu
      // item for the new style...
      WinCheckMenuItem ( hwndMenu, OldStyle, FALSE ) ;
      WinCheckMenuItem ( hwndMenu, NewStyle, TRUE ) ;
      }
   }



//----------------------------------------------------------------------------
// Name:          SourceControlDialogProc
//
// Description:   Window function for source control.
//
// Concepts:      Includes circular sliders for zoom and stretch, horz
//                and vert scroll bars for panning, readouts in upper
//                left quadrant, and button for switching between
//                zoom and stretch.
//
// Parameters:    Std dialog proc parms
//
// Returns:       Std dialog proc returns
//
//----------------------------------------------------------------------------
MRESULT EXPENTRY SourceControlDialogProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
   {
   PRECTL            prcl ;
   BOOL              fSuccess ;
   PVIDEOWINDOWINFO  pvi ;

   switch(msg)
      {
      case WM_INITDLG :

         // Initialize pointer to video window info...
         pvi = &paiMain->vwiInfo ;

         // Set initial mode to Zoom...
         pvi->fZoom = TRUE ;

         // Get a pointer to the source buffer rectangle current values...
         prcl = &pvi->rclCurSrc ;

         // Display the source buffer coordinates...
         DisplaySourceTextValues ( hwnd, prcl ) ;

         // Update the pan scroll bars to reflect the current range and position...
         UpdatePanScrollBarCharacteristics ( hwnd, pvi ) ;

         // Update the circular slider to reflect the current range and position...
         UpdateSourceZoomCharacteristics ( hwnd, pvi ) ;

         // Update the stretch scroll bars to reflect the current range and position...
         UpdateHorzStretchCharacteristics ( hwnd, pvi ) ;
         UpdateVertStretchCharacteristics ( hwnd, pvi ) ;

         return( (MRESULT) 0);


      case WM_COMMAND :

         // Get the pushbutton id...
         switch ( SHORT1FROMMP ( mp1 ) )
            {

            // User selected zoom/stretch toggle switch...
            case ID_SRC_RCL_ZOOMSTRETCH:

               // Initialize pointer to video window info...
               pvi = &paiMain->vwiInfo ;

               // Show the appropriate set of windows...
               if ( pvi->fZoom )
                  {
                  pvi->fZoom = FALSE ;
                  }
               else
                  {
                  pvi->fZoom = TRUE  ;
                  }
               ToggleSourceControlDialog ( paiMain->hwndSourceDlg, pvi ) ;
               break;

            default:
               break;

            }   // End of command switch...
         return ( MRESULT ) 0 ;


      case WM_VSCROLL :

         // Switch on the scroll window id...
         switch ( SHORT1FROMMP ( mp1 ) )
            {

            // It's the vertical *pan* scroll bar...
            case ID_SRC_RCL_PAN_VERT:

               // Initialize pointer to video window info...
               pvi = &paiMain->vwiInfo ;

               // Try to pan....
               fSuccess = AttemptVerticalPan ( hwnd, mp2, pvi ) ;

               // If successful...then update the display and
               // change the overlay...
               //
               // Note: Pan will always be successful, unless the
               //       source area is already at the max. boundary in the
               //       direction of scroll...
               //
               if ( fSuccess )
                  {
                  UpdateSourceArea ( hwnd, pvi ) ;
                  SetVerticalScrollBarPosition ( hwnd, pvi ) ;
                  }
               break ;
            }     // End of switch on vertical scroll window id....
         return ( MRESULT ) 0 ;


      case WM_HSCROLL :

         // Switch on the scroll window id...
         switch ( SHORT1FROMMP ( mp1 ) )
            {

            // It's the horizontal *pan* scroll bar...
            case ID_SRC_RCL_PAN_HORZ:

               // Initialize pointer to video window info...
               pvi = &paiMain->vwiInfo ;

               // Try to pan....
               fSuccess = AttemptHorizontalPan ( hwnd, mp2, pvi ) ;

               // If successful...then update the display and
               // change the overlay...
               //
               // Note: Pan will always be successful, unless the
               //       source area is already at the max. boundary in the
               //       direction of scroll...
               //
               if ( fSuccess )
                  {
                  UpdateSourceArea ( hwnd, pvi ) ;
                  SetHorizontalScrollBarPosition ( hwnd, pvi ) ;
                  }
               break ;
            }     // End of switch on vertical scroll window id....
         return ( MRESULT ) 0 ;


      case WM_CONTROL:

         // Initialize pointer to video window info...
         pvi = &paiMain->vwiInfo ;

         // Switch on the id...
         switch ( SHORT1FROMMP ( mp1 ) )
            {


            // Our zoom circular slider...
            case ID_SRC_RCL_ZOOM_CONTROL:

               // Switch on the message...
               switch ( SHORT2FROMMP ( mp1 ) )
                  {
                  case CSN_SETFOCUS             :
                  case CSN_QUERYBACKGROUNDCOLOR :
                     break;

                  case CSN_CHANGED:
                  case CSN_TRACKING:

                     // Try to zoom...
                     fSuccess = AttemptZoom ( hwnd, mp2, pvi ) ;

                     // If successful...then update the display and
                     // change the overlay...
                     //
                     // Note: Zoom will always be successful, unless the
                     //       source area is already at the max. boundary in the
                     //       direction of scroll...
                     //
                     if ( fSuccess )
                        {
                        UpdatePanScrollBarCharacteristics (hwnd, pvi);
                        UpdateSourceArea ( hwnd, pvi ) ;
                        }
                     break;     // End of CSN_CHANGED case...
                  }        // End of switch on the message...
               break ; // End of case ID_SRC_RCL_ZOOM_CONTROL:


            // Our horizontal stretch circular slider...
            case ID_SRC_RCL_STRETCH_HORZ:

               // Switch on the message...
               switch ( SHORT2FROMMP ( mp1 ) )
                  {
                  case CSN_SETFOCUS             :
                  case CSN_QUERYBACKGROUNDCOLOR :
                     break;


                  case CSN_CHANGED:
                  case CSN_TRACKING:

                     // Try to stretch...
                     fSuccess = AttemptStretch ( hwnd, mp2, pvi, TRUE ) ;

                     // If successful...then update the display and
                     // change the overlay...
                     //
                     // Note: Stretch will always be successful, unless the
                     //       source area is already at the max. boundary in the
                     //       direction of stretch...
                     //
                     if ( fSuccess )
                        {
                        UpdatePanScrollBarCharacteristics ( hwnd, pvi ) ;
                        UpdateSourceArea ( hwnd, pvi ) ;
                        }
                     break;     // End of CSN_CHANGED case...
                  }        // End of switch on the message...
               break ; // End of case ID_SRC_RCL_STRETCH_HORZ:


            // Our vertical stretch circular slider...
            case ID_SRC_RCL_STRETCH_VERT:

               // Switch on the message...
               switch ( SHORT2FROMMP ( mp1 ) )
                  {
                  case CSN_SETFOCUS             :
                  case CSN_QUERYBACKGROUNDCOLOR :
                     break;


                  case CSN_CHANGED:
                  case CSN_TRACKING:

                     // Try to stretch...
                     fSuccess = AttemptStretch ( hwnd, mp2, pvi, FALSE ) ;

                     // If successful...then update the display and
                     // change the overlay...
                     //
                     // Note: Stretch will always be successful, unless the
                     //       source area is already at the max. boundary in the
                     //       direction of stretch...
                     //
                     if ( fSuccess )
                        {
                        UpdatePanScrollBarCharacteristics ( hwnd, pvi ) ;
                        UpdateSourceArea ( hwnd, pvi ) ;
                        }
                     break;     // End of CSN_CHANGED case...
                  }        // End of switch on the message...
               break ; // End of case ID_SRC_RCL_STRETCH_HORZ:


            }     // End of switch on the control id...
         return ( MRESULT ) 0 ;   // End of WM_CONTROL case...


      case WM_CLOSE:

         // Zero our copy of the handle...
         paiMain->hwndSourceDlg =  ( HWND ) 0 ;
         return ( WinDefDlgProc ( hwnd, msg, mp1, mp2 ) ) ;


      case WM_DESTROY:

         // Zero our copy of the handle...
         paiMain->hwndSourceDlg =  ( HWND ) 0 ;
         return  ( MRESULT ) 0 ;


      default:
         return ( WinDefDlgProc ( hwnd, msg, mp1, mp2 ) ) ;
       }
   return ( MRESULT ) 0 ;
   }


//----------------------------------------------------------------------------
// Name:          ToggleSourceControlDialog
//
// Description:   Switch between stretch mode and zoom mode.
//
// Concepts:      In a real-life application overlay management wd
//                probably not be so complex; but here we are trying
//                to manage 2 different modes of adjustments (stretch
//                and zoom), applying several restrictions( source rect
//                cannot be bigger than source buffer nor smaller than
//                destination rectangle; during zoom, source and dest
//                rects must have same aspect ratio), and managing
//                several controls and readouts.
//
// Parameters:    hwnd - the handle of Source Area Control dialog
//
//                pvi - Pointer to a VIDVIEW-specific structure of
//                      associated video parameters.
//
// Returns:       VOID
//
//----------------------------------------------------------------------------
SZ szStretch [] = "Stretch" ;
SZ szZoom    [] = "Zoom"    ;
VOID ToggleSourceControlDialog ( HWND hwnd, PVIDEOWINDOWINFO pvi )
   {
   PLONG    pHide, pShow, p ;
   LONG     i, Width, Height, LoBound, HiBound ;
   LONG     XOrgWnd, YOrgWnd, XOrgText, YOrgText, XOrgCy, YOrgCy ;
   RECTL    rclResult1, rclResult2 ;
   PRECTL   prclR1 ;
   BOOL     fSuccess = TRUE ;
   PSZ      pszPushButtonText ;
   RECTL    rclTemp ;
   POINTL   ptlMap ;
   PPOINTL  pptl ;


   // If we are switching to zoom...
   if ( pvi->fZoom )
      {
      // Arrange to hide the stretch windows and show the zoom windows...
      pHide = aStretchWindowIDs ;
      pShow = aZoomWindowIDs    ;

      // Get access to data...
      rclResult1 = pvi->rclCurSrc ;
      prclR1    = &rclResult1 ;
      LoBound    = pvi->SrcZoomLoBound ;
      HiBound    = pvi->SrcZoomHiBound ;

      // If x limits the max. size...
      if ( pvi->fSrcZoomXLimitsMaxSize )
         {

         // Get the width...clip to current bounds...
         Width  = ( prclR1->xRight - prclR1->xLeft   ) + 1 ;
         if ( Width < LoBound )
            {
            Width = LoBound ;
            }
         else if ( Width > HiBound )
            {
            Width = HiBound ;
            }

         // Make the height a function of the width...
         Height = ( ( Width * pvi->SrcAspectY ) + ( pvi->SrcAspectX / 2 ) )
                    / pvi->SrcAspectX ;
         }

      // else y limits the max size...
      else
         {
         // Get the height...clip to current bounds...
         Height = ( prclR1->yTop   - prclR1->yBottom ) + 1 ;
         if ( Height < LoBound )
            {
            Height = LoBound ;
            }
         else if ( Height > HiBound )
            {
            Height = HiBound ;
            }


         // Make the width  a function of the height...
         Width = ( ( Height * pvi->SrcAspectX ) + ( pvi->SrcAspectY / 2 ) )
                   / pvi->SrcAspectY ;
         }

      // Set the new rectangle coordinates...
      prclR1->xRight = prclR1->xLeft   + ( Width  - 1 ) ;
      prclR1->yTop   = prclR1->yBottom + ( Height - 1 ) ;

      // Fit the correct-aspect-ratio rectangle into the video source buffer...
      fSuccess = FitRectInsideRect (
                       &rclResult1,
                       &rclResult2,
                       &pvi->rclMinSrc,
                       &pvi->rclMaxSrc ) ;

      // If the fit was successful...
      // ...Copy the change into the current source rectangle...
      // and return success...
      if ( fSuccess )
         {
         pvi->rclCurSrc = rclResult2 ;
         }

      // Update the zoom control characteristics to reflect the
      // current range and position...
      UpdateSourceZoomCharacteristics ( hwnd, pvi ) ;
      }

   // Else we are to stretch...
   else
      {
      // Arrange to hide the zoom windows and show the stretch windows...
      pHide = aZoomWindowIDs    ;
      pShow = aStretchWindowIDs ;

      // Update the stretch scroll bars to reflect the current range and position...
      UpdateHorzStretchCharacteristics ( hwnd, pvi ) ;
      UpdateVertStretchCharacteristics ( hwnd, pvi ) ;
      }

   // If successful...
   if ( fSuccess )
      {

      // Get the current window size...and map to desktop coords...
      WinQueryWindowRect ( hwnd, &rclTemp ) ;
      Width  = rclTemp.xRight ;
      Height = rclTemp.yTop ;
      WinMapWindowPoints ( hwnd, HWND_DESKTOP, ( PPOINTL ) &rclTemp, 2 ) ;
      XOrgWnd = rclTemp.xLeft   ;
      YOrgWnd = rclTemp.yBottom ;

      // Initialize changes if we have not already done so...
      // ...We do this only once per existence of the source control dlg...
      // ...These are the values used when changing from the Zoom
      // mode to the Stretch mode...We have to:
      //    -- Determine the difference in the height of the dialog window
      //    -- Determine the (x, y) of the "cy" control for Zoom and Stretch
      //    -- Determine the (x, y) of the "Vert Pan" control for Zoom and Stretch
      if ( pvi->HeightDifference == 0 )
         {
         // Determine differnce in height..
         pptl    = &ptlMap ;
         pptl->x =  0 ;
         pptl->y = 10 ;
         WinMapDlgPoints ( hwnd, pptl, 1, TRUE ) ;
         pvi->HeightDifference = pptl->y ;

         // Determ cy and text locations for zoom...
         pptl    = &pvi->OrgCyForZoom ;
         pptl->x = 58 ;
         pptl->y = 61 ;
         WinMapDlgPoints ( hwnd, pptl, 1, TRUE ) ;
         pptl    = &pvi->OrgTextForZoom ;
         pptl->x = 70 ;
         pptl->y = 73 ;
         WinMapDlgPoints ( hwnd, pptl, 1, TRUE ) ;

         // Determ cy and text locations for stretch
         pptl    = &pvi->OrgCyForStretch ;
         pptl->x = 16 ;
         pptl->y = 84 ;
         WinMapDlgPoints ( hwnd, pptl, 1, TRUE ) ;
         pptl    = &pvi->OrgTextForStretch ;
         pptl->x = 88 ;
         pptl->y = 84 ;
         WinMapDlgPoints ( hwnd, pptl, 1, TRUE ) ;
         }


      // If switching to zoom...
      if ( pvi->fZoom )
         {
         // Make the pushbutton say "stretch"...
         pszPushButtonText = szStretch ;

         // Get the values for this mode...
         Height  -= pvi->HeightDifference    ;
         YOrgWnd += pvi->HeightDifference    ;
         XOrgCy   = pvi->OrgCyForZoom.x      ;
         YOrgCy   = pvi->OrgCyForZoom.y      ;
         XOrgText = pvi->OrgTextForZoom.x    ;
         YOrgText = pvi->OrgTextForZoom.y    ;
         }
      // Else switching to stretch...
      else
         {
         // Make the pushbutton say "zoom"...
         pszPushButtonText = szZoom ;

         // Get the values for this mode...
         Height  += pvi->HeightDifference    ;
         YOrgWnd -= pvi->HeightDifference    ;
         XOrgCy   = pvi->OrgCyForStretch.x   ;
         YOrgCy   = pvi->OrgCyForStretch.y   ;
         XOrgText = pvi->OrgTextForStretch.x ;
         YOrgText = pvi->OrgTextForStretch.y ;
         }

      // Change the text of the push button...
      WinSetDlgItemText ( hwnd, ID_SRC_RCL_ZOOMSTRETCH, pszPushButtonText ) ;

      // Change the height and position of the window...
      WinSetWindowPos ( hwnd, 0L, XOrgWnd, YOrgWnd, Width, Height,
                        SWP_SIZE|SWP_MOVE );

      // Move the controls...
      WinSetWindowPos ( WinWindowFromID (hwnd, ID_SRC_RCL_TEXT_CY), 0L,
                        XOrgCy, YOrgCy, 0, 0, SWP_MOVE ) ;
      WinSetWindowPos ( WinWindowFromID (hwnd, ID_SRC_RCL_PAN_V_LABEL), 0L,
                        XOrgText, YOrgText, 0, 0, SWP_MOVE ) ;

      // Do the hides...
      for ( i = 0, p = pHide; i < 3; i++, p++ )
         {
         if ( ( LONG ) *p != 0 )
            {
            WinSetWindowPos ( WinWindowFromID ( hwnd, *p ), 0,0,0,0,0, SWP_HIDE );
            }
         }

      // Update the pan scroll bars and the readouts...
      UpdatePanScrollBarCharacteristics (hwnd, pvi );
      UpdateSourceArea ( hwnd, pvi ) ;

      // Do the shows...
      for ( i = 0, p = pShow; i < 3; i++, p++ )
         {
         if ( ( LONG ) *p != 0 )
            {
            WinSetWindowPos ( WinWindowFromID ( hwnd, *p ), 0,0,0,0,0, SWP_SHOW );
            }
         }
      }
   }


//*****************************************************************************
//***************************************************************************** */
//***
//*** MCI Calls Section
//***    -- Overlay calls...
//***    -- General calls...   ==> see VIDVIEW.C
//***    -- Videodisc calls... ==> see VIDVIEW.C
//***
//*****************************************************************************
//*****************************************************************************
//-----------------------------------------------------------------------------
// Name: OvlSetVideoWindow()
//
// Description:   Tells overlay device to make the specified window the
//                video window.
//
// Concepts:      Sets window to which the overlay device will direct
//                video.  Thereafter the overlay device manages video
//                update when the window is moved or uncovered.  Application
//                is resp for correct aspect ratio (size and shape of
//                window, or of destination rectangle within window, and
//                keeping the destination painted CLR_PINK so that
//                video will show through.  The window class has to be
//                registered CS_SIZEREDRAW and CS_MOVENOTIFY.
//
// Parameters:    wDeviceID - the overlay device
//
//                hwndAlternate - the new video window
//
// Returns:       Return code from mciSendCommand()
//
//-----------------------------------------------------------------------------
DWORD  OvlSetVideoWindow (WORD wDeviceID, HWND hwndAlternate)
   {
   WORD    wUserParm = 0;
   DWORD   dwReturnCode;
   MCI_VID_WINDOW_PARMS mciWindowParms;

   memset (&mciWindowParms, 0x00, sizeof (MCI_VID_WINDOW_PARMS));
   mciWindowParms.dwCallback = (DWORD)hwndNotify;
   mciWindowParms.hwndDest = hwndAlternate;

   dwReturnCode = mciSendCommand ( wDeviceID, MCI_WINDOW,
                                   MCI_WAIT | MCI_OVLY_WINDOW_HWND,
                                   ( DWORD )&mciWindowParms, wUserParm ) ;
   return dwReturnCode ;
   }

//----------------------------------------------------------------------------
// Name:          OvlPutSrcRect ()
//
// Description:   Sets source area rectangle size and shape.
//
// Concepts:      Source buffer is conceptual 706 by 484 buffer containing
//                incoming source image.  This call determines what rectangle
//                with this source buffer will be shown.
//
// Parameters:    wDeviceID - overlay device id
//
//                prcl - source rectangle
//
//                hwndCallback - handle for MCI_NOTIFY notification.
//
// Returns:       Return code from mciSendCommand()
//
//----------------------------------------------------------------------------
DWORD OvlPutSrcRect ( WORD wDeviceID, PRECTL prcl, HWND hwndCallback )
   {
   MCI_OVLY_RECT_PARMS mciRectParms;

   memset ( &mciRectParms, 0x00, sizeof ( MCI_OVLY_RECT_PARMS ) );
   mciRectParms.dwCallback = ( DWORD ) hwndCallback ;
   mciRectParms.rc         = *prcl ;

   return mciSendCommand ( wDeviceID,
                           MCI_PUT,
                           MCI_OVLY_PUT_RECT | MCI_OVLY_PUT_SOURCE | MCI_WAIT,
                           ( DWORD ) &mciRectParms,
                           0 );
   }


//----------------------------------------------------------------------------
// Name:          OvlPutDstRect
//
// Description:   Sets destination rectangle size and shape.
//
// Concepts:      The input rectangle is in coordinates relative to the
//                current video window.
//
//                Example:
//
//                a) Video window is at
//                   xLeft,yBottom, xRight,yTop=(10,10, 181,129)
//                   relative to its parent.
//
//                   Relative to itself, the video window is at
//                   (0,0, 172,119).
//
//                b) The application calls OvlPutDstRect() with the
//                   rectangle (0,0, 172,119).
//
//                c) The video destination rectangle overlays the
//                   video window and is the same size.
//
//                Notes:
//
//                #1. Notice that some PM functions, such as
//                    WinQueryWindowRect(), return a rectangle that is
//                    1 pel larger in height and width than the
//                    actual window.
//
//                    For example, if WinQueryWindowRect() were called with
//                    the hwnd of the video window in the example above,
//                    it wd return the rectangle (0,0, 172,120).
//
//                #2. Notice that the application may use ONLY PART of the
//                    video window as the video destination, by specifying
//                    a sub-rectangle of the video window in OvlPutDstRect().
//
//                #3. For video to be seen, the destination rect must be
//                    painted with CLR_PINK.
//
//
// Parameters:    wDeviceID - overlay device id
//
//                prcl - source rectangle
//
//                hwndCallback - handle for MCI_NOTIFY notification.
//
// Returns:       Return code from mciSendCommand()
//
//----------------------------------------------------------------------------
DWORD OvlPutDstRect ( WORD wDeviceID, PRECTL prcl, HWND hwndCallback )
   {
   MCI_OVLY_RECT_PARMS mciRectParms;

   memset ( &mciRectParms, 0x00, sizeof ( MCI_OVLY_RECT_PARMS ) );
   mciRectParms.dwCallback = ( DWORD ) hwndCallback ;
   mciRectParms.rc         = *prcl ;

   return mciSendCommand ( wDeviceID,
                           MCI_PUT,
                           MCI_OVLY_PUT_RECT | MCI_OVLY_PUT_DESTINATION | MCI_WAIT,
                           ( DWORD ) &mciRectParms,
                           0 );
   }


//----------------------------------------------------------------------------
// Name:          OvlGetSrcExtRect
//
// Description:   Gets extent of maximum source rectangle size.
//
// Concepts:      Uses MCI_GETDEVCAPS calls to get the maximum
//                width and height of the video source buffer.
//
// Parameters:    wDeviceID - overlay device id
//
//                hwndCallback - handle for MCI_NOTIFY notification.
//
//                pdwXExtent, pdwYExtent - pointers to return values;
//                     the CALLER must make sure that these pointers
//                     point to valid DWORDS that the caller owns.
//
// Returns:       Return code from mciSendCommand()
//
//----------------------------------------------------------------------------
DWORD OvlGetSrcExtRect ( WORD wDeviceID, HWND hwndCallback,
                      DWORD *pdwXExtent, DWORD *pdwYExtent )
   {
   MCI_GETDEVCAPS_PARMS mciDevCapsParms;
   DWORD            dwReturn ;

   // Initialize...
   *pdwXExtent = 0 ;
   *pdwYExtent = 0 ;

   // Get the x value...
   memset ( &mciDevCapsParms, 0x00, sizeof ( MCI_GETDEVCAPS_PARMS ) );
   mciDevCapsParms.dwCallback = ( DWORD ) hwndCallback ;
   mciDevCapsParms.dwItem     = MCI_OVLY_GETDEVCAPS_VIDEO_X_EXTENT;
   dwReturn = mciSendCommand ( wDeviceID,
                               MCI_GETDEVCAPS,
                               MCI_GETDEVCAPS_ITEM | MCI_WAIT,
                               ( DWORD ) &mciDevCapsParms,
                               0 ) ;
   if ( LOUSHORT ( dwReturn ) != MCIERR_SUCCESS )
      {
      return dwReturn ;
      }
   *pdwXExtent = mciDevCapsParms.dwReturn ;

   // Get the y value...
   memset ( &mciDevCapsParms, 0x00, sizeof ( MCI_GETDEVCAPS_PARMS ) );
   mciDevCapsParms.dwCallback = ( DWORD ) hwndCallback ;
   mciDevCapsParms.dwItem     = MCI_OVLY_GETDEVCAPS_VIDEO_Y_EXTENT;
   dwReturn = mciSendCommand ( wDeviceID,
                               MCI_GETDEVCAPS,
                               MCI_GETDEVCAPS_ITEM | MCI_WAIT,
                               ( DWORD ) &mciDevCapsParms,
                               0 ) ;
   if ( LOUSHORT ( dwReturn ) != MCIERR_SUCCESS )
      {
      return dwReturn ;
      }
   *pdwYExtent = mciDevCapsParms.dwReturn;
   return dwReturn ;
   }


//----------------------------------------------------------------------------
// Name:          OvlGetDstRect
//
// Description:   Get destination rectangle position.
//
// Concepts:      The rectangle returned contains the coordinates
//                of the video destination RELATIVE TO THE VIDEO WINDOW.
//
//                The destination rectangle may cover the entire video
//                window or a portion of it.  Active video will show
//                through at the destination only where it is covered
//                with the color CLR_PINK.
//
//                See OvlPutDstRect().
//
// Parameters:    wDeviceID - overlay device id
//
//                hwndCallback - handle for MCI_NOTIFY notification.
//
//                prcl - pointer to return rectangle
//                     the CALLER must make sure that these pointers
//                     point to valid RECTL that the caller owns.
//
// Returns:       Return code from mciSendCommand()
//
//----------------------------------------------------------------------------
DWORD OvlGetDstRect  ( WORD wDeviceID, HWND hwndCallback, PRECTL prcl )
   {
   MCI_OVLY_RECT_PARMS mciRectParms ;
   DWORD            dwReturn ;

   // Get the values...
   memset ( &mciRectParms, 0x00, sizeof ( MCI_OVLY_RECT_PARMS ) );

   mciRectParms.dwCallback = ( DWORD ) hwndCallback ;
   dwReturn = mciSendCommand ( wDeviceID,
                               MCI_WHERE,
                               MCI_OVLY_WHERE_DESTINATION | MCI_WAIT,
                               ( DWORD ) &mciRectParms,
                               0 ) ;
   if ( LOUSHORT ( dwReturn ) != MCIERR_SUCCESS )
      {
      return dwReturn ;
      }

   // Store the rect size...
   prcl->xLeft    = mciRectParms.rc.xLeft  ;
   prcl->xRight   = mciRectParms.rc.xRight ;
   prcl->yBottom  = mciRectParms.rc.yBottom;
   prcl->yTop     = mciRectParms.rc.yTop   ;

   return dwReturn ;
   }


//----------------------------------------------------------------------------
// Name:          OvlGetSrcRect
//
// Description:   Get source rectangle position.
//
// Concepts:      Returns current source rect dimensions within video
//                source buffer.  May be called to determine current
//                value when application starts up.
//
//                See OvlPutSrcRect().
//
// Parameters:    wDeviceID - overlay device id
//
//                hwndCallback - handle for MCI_NOTIFY notification.
//
//                prcl - pointer to return rectangle
//                     the CALLER must make sure that these pointers
//                     point to valid RECTL that the caller owns.
//
// Returns:       Return code from mciSendCommand()
//
//----------------------------------------------------------------------------
DWORD OvlGetSrcRect  ( WORD wDeviceID, HWND hwndCallback, PRECTL prcl )
   {
   MCI_OVLY_RECT_PARMS mciRectParms ;
   DWORD            dwReturn ;

   // Get the values...
   memset ( &mciRectParms, 0x00, sizeof ( MCI_OVLY_RECT_PARMS ) );

   mciRectParms.dwCallback = ( DWORD ) hwndCallback ;
   dwReturn = mciSendCommand ( wDeviceID,
                               MCI_WHERE,
                               MCI_OVLY_WHERE_SOURCE | MCI_WAIT,
                               ( DWORD ) &mciRectParms,
                               0 ) ;
   if ( LOUSHORT ( dwReturn ) != MCIERR_SUCCESS )
      {
      return dwReturn ;
      }

   // Store the rect size...
   prcl->xLeft    = mciRectParms.rc.xLeft  ;
   prcl->xRight   = mciRectParms.rc.xRight ;
   prcl->yBottom  = mciRectParms.rc.yBottom;
   prcl->yTop     = mciRectParms.rc.yTop   ;

   return dwReturn ;
   }


//----------------------------------------------------------------------------
// Name:          OvlSetInput
//
// Description:   Sets current video input.
//
// Concepts:      In VIDVIEW this function is used only to set the
//                MCI_VIDEO_IN_CONNECTOR to 1, 2, or 3, corresponding
//                to input 1, 2, or 3 from the M-Motion Video Adapter/A.
//
// Parameters:    wDeviceID - overlay device id
//
//                dwSourceNum - 1, 2, or 3
//
//                dwSourceType - MCI_VIDEO_IN_CONNECTOR
//
// Returns:       Return code from mciSendCommand()
//
//----------------------------------------------------------------------------
DWORD OvlSetInput (WORD wDeviceID,DWORD dwSourceNum,DWORD dwSourceType)
   {
   MCI_CONNECTOR_PARMS  mciConnectorParms;
   DWORD dwReturn ;

   mciConnectorParms.dwCallback       = 0            ;
   mciConnectorParms.dwReturn         = 0            ;
   mciConnectorParms.dwConnectorType  = dwSourceType ;
   mciConnectorParms.dwConnectorIndex = dwSourceNum  ;

   dwReturn = mciSendCommand (
                  wDeviceID, MCI_CONNECTOR,
                  MCI_WAIT | MCI_ENABLE_CONNECTOR |
                  MCI_CONNECTOR_TYPE | MCI_CONNECTOR_INDEX,
                  ( DWORD )&mciConnectorParms,
                  0 ) ;

   return dwReturn ;
   }

//*****************************************************************************
// End of file
//*****************************************************************************
